@@ -58,7 +58,15 @@
 #include "storage/storage.h"
 #include "utils.h"
 
-lxc_log_define(lxc_cgfsng, lxc);
+#ifndef HAVE_STRLCPY
+#include "include/strlcpy.h"
+#endif
+
+#ifndef HAVE_STRLCAT
+#include "include/strlcat.h"
+#endif
+
+lxc_log_define(cgfsng, cgroup);
 
 static void free_string_list(char **clist)
 {
@@ -266,14 +274,13 @@ static uint32_t *lxc_cpumask(char *buf, size_t nbits)
 	char *token;
 	size_t arrlen;
 	uint32_t *bitarr;
-	char *saveptr = NULL;
 
 	arrlen = BITS_TO_LONGS(nbits);
 	bitarr = calloc(arrlen, sizeof(uint32_t));
 	if (!bitarr)
 		return NULL;
 
-	for (; (token = strtok_r(buf, ",", &saveptr)); buf = NULL) {
+	lxc_iterate_parts(token, buf, ",") {
 		errno = 0;
 		unsigned end, start;
 		char *range;
@@ -390,7 +397,7 @@ static bool cg_legacy_filter_and_set_cpus(char *path, bool am_initialized)
 
 	/* Get maximum number of cpus found in possible cpuset. */
 	maxposs = get_max_cpus(posscpus);
-	if (maxposs < 0)
+	if (maxposs < 0 || maxposs >= INT_MAX - 1)
 		goto on_error;
 
 	if (!file_exists(__ISOL_CPUS)) {
@@ -435,7 +442,7 @@ static bool cg_legacy_filter_and_set_cpus(char *path, bool am_initialized)
 
 	/* Get maximum number of cpus found in isolated cpuset. */
 	maxisol = get_max_cpus(isolcpus);
-	if (maxisol < 0)
+	if (maxisol < 0 || maxisol >= INT_MAX - 1)
 		goto on_error;
 
 	if (maxposs < maxisol)
@@ -687,8 +694,7 @@ static bool controller_found(struct hierarchy **hlist, char *entry)
  */
 static bool all_controllers_found(struct cgroup_ops *ops)
 {
-	char *p;
-	char *saveptr = NULL;
+	char **cur;
 	struct hierarchy **hlist = ops->hierarchies;
 
 	if (!controller_found(hlist, "freezer")) {
@@ -699,9 +705,9 @@ static bool all_controllers_found(struct cgroup_ops *ops)
 	if (!ops->cgroup_use)
 		return true;
 
-	for (; (p = strtok_r(ops->cgroup_use, ",", &saveptr)); ops->cgroup_use = NULL)
-		if (!controller_found(hlist, p)) {
-			ERROR("No %s controller mountpoint found", p);
+	for (cur = ops->cgroup_use; cur && *cur; cur++)
+		if (!controller_found(hlist, *cur)) {
+			ERROR("No %s controller mountpoint found", *cur);
 			return false;
 		}
 
@@ -721,7 +727,7 @@ static char **cg_hybrid_get_controllers(char **klist, char **nlist, char *line,
 	 */
 	int i;
 	char *dup, *p2, *tok;
-	char *p = line, *saveptr = NULL, *sep = ",";
+	char *p = line, *sep = ",";
 	char **aret = NULL;
 
 	for (i = 0; i < 4; i++) {
@@ -748,16 +754,17 @@ static char **cg_hybrid_get_controllers(char **klist, char **nlist, char *line,
 	*p2 = '\0';
 
 	if (type == CGROUP_SUPER_MAGIC) {
-		/* strdup() here for v1 hierarchies. Otherwise strtok_r() will
-		 * destroy mountpoints such as "/sys/fs/cgroup/cpu,cpuacct".
+		/* strdup() here for v1 hierarchies. Otherwise
+		 * lxc_iterate_parts() will destroy mountpoints such as
+		 * "/sys/fs/cgroup/cpu,cpuacct".
 		 */
 		dup = strdup(p);
 		if (!dup)
 			return NULL;
 
-		for (tok = strtok_r(dup, sep, &saveptr); tok;
-		     tok = strtok_r(NULL, sep, &saveptr))
+		lxc_iterate_parts(tok, dup, sep) {
 			must_append_controller(klist, nlist, &aret, tok);
+		}
 
 		free(dup);
 	}
@@ -779,15 +786,14 @@ static char **cg_unified_make_empty_controller(void)
 static char **cg_unified_get_controllers(const char *file)
 {
 	char *buf, *tok;
-	char *saveptr = NULL, *sep = " \t\n";
+	char *sep = " \t\n";
 	char **aret = NULL;
 
 	buf = read_file(file);
 	if (!buf)
 		return NULL;
 
-	for (tok = strtok_r(buf, sep, &saveptr); tok;
-	     tok = strtok_r(NULL, sep, &saveptr)) {
+	lxc_iterate_parts(tok, buf, sep) {
 		int newentry;
 		char *copy;
 
@@ -871,7 +877,7 @@ static char *copy_to_eol(char *p)
  */
 static bool controller_in_clist(char *cgline, char *c)
 {
-	char *tok, *saveptr = NULL, *eol, *tmp;
+	char *tok, *eol, *tmp;
 	size_t len;
 
 	eol = strchr(cgline, ':');
@@ -883,8 +889,7 @@ static bool controller_in_clist(char *cgline, char *c)
 	memcpy(tmp, cgline, len);
 	tmp[len] = '\0';
 
-	for (tok = strtok_r(tmp, ",", &saveptr); tok;
-	     tok = strtok_r(NULL, ",", &saveptr)) {
+	lxc_iterate_parts(tok, tmp, ",") {
 		if (strcmp(tok, c) == 0)
 			return true;
 	}
@@ -948,7 +953,7 @@ static int get_existing_subsystems(char ***klist, char ***nlist)
 		return -1;
 
 	while (getline(&line, &len, f) != -1) {
-		char *p, *p2, *tok, *saveptr = NULL;
+		char *p, *p2, *tok;
 		p = strchr(line, ':');
 		if (!p)
 			continue;
@@ -970,8 +975,7 @@ static int get_existing_subsystems(char ***klist, char ***nlist)
 			continue;
 		}
 
-		for (tok = strtok_r(p, ",", &saveptr); tok;
-		     tok = strtok_r(NULL, ",", &saveptr)) {
+		lxc_iterate_parts(tok, p, ",") {
 			if (strncmp(tok, "name=", 5) == 0)
 				must_append_string(nlist, tok);
 			else
@@ -1032,62 +1036,6 @@ static void lxc_cgfsng_print_basecg_debuginfo(char *basecginfo, char **klist,
 		TRACE("named subsystem %d: %s", k, *it);
 }
 
-static int recursive_destroy(char *dirname)
-{
-	int ret;
-	struct dirent *direntp;
-	DIR *dir;
-	int r = 0;
-
-	dir = opendir(dirname);
-	if (!dir)
-		return -1;
-
-	while ((direntp = readdir(dir))) {
-		char *pathname;
-		struct stat mystat;
-
-		if (!strcmp(direntp->d_name, ".") ||
-		    !strcmp(direntp->d_name, ".."))
-			continue;
-
-		pathname = must_make_path(dirname, direntp->d_name, NULL);
-
-		ret = lstat(pathname, &mystat);
-		if (ret < 0) {
-			if (!r)
-				WARN("Failed to stat \"%s\"", pathname);
-			r = -1;
-			goto next;
-		}
-
-		if (!S_ISDIR(mystat.st_mode))
-			goto next;
-
-		ret = recursive_destroy(pathname);
-		if (ret < 0)
-			r = -1;
-	next:
-		free(pathname);
-	}
-
-	ret = rmdir(dirname);
-	if (ret < 0) {
-		if (!r)
-			WARN("%s - Failed to delete \"%s\"", strerror(errno), dirname);
-		r = -1;
-	}
-
-	ret = closedir(dir);
-	if (ret < 0) {
-		if (!r)
-			WARN("%s - Failed to delete \"%s\"", strerror(errno), dirname);
-		r = -1;
-	}
-
-	return r;
-}
-
 static int cgroup_rmdir(struct hierarchy **hierarchies,
 			const char *container_cgroup)
 {
@@ -1195,19 +1143,23 @@ static bool cg_unified_create_cgroup(struct hierarchy *h, char *cgname)
 	 * some thinking.
 	 */
 	for (it = h->controllers; it && *it; it++) {
-                full_len += strlen(*it) + 2;
-                add_controllers = must_realloc(add_controllers, full_len + 1);
-                if (h->controllers[0] == *it)
-                        add_controllers[0] = '\0';
-                strcat(add_controllers, "+");
-                strcat(add_controllers, *it);
-                if ((it + 1) && *(it + 1))
-                        strcat(add_controllers, " ");
+		full_len += strlen(*it) + 2;
+		add_controllers = must_realloc(add_controllers, full_len + 1);
+
+		if (h->controllers[0] == *it)
+			add_controllers[0] = '\0';
+
+		(void)strlcat(add_controllers, "+", full_len + 1);
+		(void)strlcat(add_controllers, *it, full_len + 1);
+
+		if ((it + 1) && *(it + 1))
+			(void)strlcat(add_controllers, " ", full_len + 1);
 	}
 
 	parts = lxc_string_split(cgname, '/');
 	if (!parts)
 		goto on_error;
+
 	parts_len = lxc_array_len((void **)parts);
 	if (parts_len > 0)
 		parts_len--;
@@ -1301,9 +1253,10 @@ static inline bool cgfsng_create(struct cgroup_ops *ops,
 		ERROR("Failed expanding cgroup name pattern");
 		return false;
 	}
+
 	len = strlen(tmp) + 5; /* leave room for -NNN\0 */
 	container_cgroup = must_alloc(len);
-	strcpy(container_cgroup, tmp);
+	(void)strlcpy(container_cgroup, tmp, len);
 	free(tmp);
 	offset = container_cgroup + len - 5;
 
@@ -1385,15 +1338,13 @@ static int chowmod(char *path, uid_t chown_uid, gid_t chown_gid,
 
 	ret = chown(path, chown_uid, chown_gid);
 	if (ret < 0) {
-		WARN("%s - Failed to chown(%s, %d, %d)", strerror(errno), path,
-		     (int)chown_uid, (int)chown_gid);
+		SYSWARN("Failed to chown(%s, %d, %d)", path, (int)chown_uid, (int)chown_gid);
 		return -1;
 	}
 
 	ret = chmod(path, chmod_mode);
 	if (ret < 0) {
-		WARN("%s - Failed to chmod(%s, %d)", strerror(errno), path,
-		     (int)chmod_mode);
+		SYSWARN("Failed to chmod(%s, %d)", path, (int)chmod_mode);
 		return -1;
 	}
 
@@ -1748,9 +1699,6 @@ static int recursive_count_nrtasks(char *dirname)
 	while ((direntp = readdir(dir))) {
 		struct stat mystat;
 
-		if (!direntp)
-			break;
-
 		if (!strcmp(direntp->d_name, ".") ||
 		    !strcmp(direntp->d_name, ".."))
 			continue;
@@ -1945,7 +1893,7 @@ static int __cg_unified_attach(const struct hierarchy *h, const char *name,
 		if (ret < 0 && errno != EEXIST)
 			goto on_error;
 
-		strcat(full_path, "/cgroup.procs");
+		(void)strlcat(full_path, "/cgroup.procs", len + 1);
 		ret = lxc_write_to_file(full_path, pidstr, len, false, 0666);
 		if (ret == 0)
 			goto on_success;
@@ -2025,7 +1973,8 @@ static int cgfsng_get(struct cgroup_ops *ops, const char *filename, char *value,
 
 	controller_len = strlen(filename);
 	controller = alloca(controller_len + 1);
-	strcpy(controller, filename);
+	(void)strlcpy(controller, filename, controller_len + 1);
+
 	p = strchr(controller, '.');
 	if (p)
 		*p = '\0';
@@ -2062,7 +2011,8 @@ static int cgfsng_set(struct cgroup_ops *ops, const char *filename,
 
 	controller_len = strlen(filename);
 	controller = alloca(controller_len + 1);
-	strcpy(controller, filename);
+	(void)strlcpy(controller, filename, controller_len + 1);
+
 	p = strchr(controller, '.');
 	if (p)
 		*p = '\0';
@@ -2179,7 +2129,8 @@ static int cg_legacy_set_data(struct cgroup_ops *ops, const char *filename,
 
 	len = strlen(filename);
 	controller = alloca(len + 1);
-	strcpy(controller, filename);
+	(void)strlcpy(controller, filename, len + 1);
+
 	p = strchr(controller, '.');
 	if (p)
 		*p = '\0';
@@ -2296,6 +2247,34 @@ static bool cgfsng_setup_limits(struct cgroup_ops *ops, struct lxc_conf *conf,
 	return __cg_unified_setup_limits(ops, &conf->cgroup2);
 }
 
+static bool cgroup_use_wants_controllers(const struct cgroup_ops *ops,
+				       char **controllers)
+{
+	char **cur_ctrl, **cur_use;
+
+	if (!ops->cgroup_use)
+		return true;
+
+	for (cur_ctrl = controllers; cur_ctrl && *cur_ctrl; cur_ctrl++) {
+		bool found = false;
+
+		for (cur_use = ops->cgroup_use; cur_use && *cur_use; cur_use++) {
+			if (strcmp(*cur_use, *cur_ctrl) != 0)
+				continue;
+
+			found = true;
+			break;
+		}
+
+		if (found)
+			continue;
+
+		return false;
+	}
+
+	return true;
+}
+
 /* At startup, parse_hierarchies finds all the info we need about cgroup
  * mountpoints and current cgroups, and stores it in @d.
  */
@@ -2411,6 +2390,10 @@ static bool cg_hybrid_init(struct cgroup_ops *ops)
 			}
 		}
 
+		/* Exclude all controllers that cgroup use does not want. */
+		if (!cgroup_use_wants_controllers(ops, controller_list))
+			goto next;
+
 		new = add_hierarchy(&ops->hierarchies, controller_list, mountpoint, base_cgroup, type);
 		if (type == CGROUP2_SUPER_MAGIC && !ops->unified)
 			ops->unified = new;
@@ -2543,8 +2526,18 @@ static bool cg_init(struct cgroup_ops *ops)
 	const char *tmp;
 
 	tmp = lxc_global_config_value("lxc.cgroup.use");
-	if (tmp)
-		ops->cgroup_use = must_copy_string(tmp);
+	if (tmp) {
+		char *chop, *cur, *pin;
+
+		pin = must_copy_string(tmp);
+		chop = pin;
+
+		lxc_iterate_parts(cur, chop, ",") {
+			must_append_string(&ops->cgroup_use, cur);
+		}
+
+		free(pin);
+	}
 
 	ret = cg_unified_init(ops);
 	if (ret < 0)
