@@ -33,7 +33,6 @@
 #include <sys/stat.h>
 #include <string.h>
 #include <pthread.h>
-#include <time.h>
 
 #include <syslog.h>
 #include <stdio.h>
@@ -66,7 +65,7 @@ static char log_prefix[LXC_LOG_PREFIX_SIZE] = "lxc";
 static char *log_fname = NULL;
 static char *log_vmname = NULL;
 
-lxc_log_define(lxc_log, lxc);
+lxc_log_define(log, lxc);
 
 static int lxc_log_priority_to_syslog(int priority)
 {
@@ -127,13 +126,13 @@ static int log_append_syslog(const struct lxc_log_appender *appender,
 	}
 
 	syslog(lxc_log_priority_to_syslog(event->priority),
-		"%s%s %s - %s:%s:%d - %s" ,
-		log_container_name ? log_container_name : "",
-		log_container_name ? ":" : "",
-		event->category,
-		event->locinfo->file, event->locinfo->func,
-		event->locinfo->line,
-		msg);
+	       "%s%s %s - %s:%s:%d - %s" ,
+	       log_container_name ? log_container_name : "",
+	       log_container_name ? ":" : "",
+	       event->category,
+	       event->locinfo->file, event->locinfo->func,
+	       event->locinfo->line,
+	       msg);
 	free(msg);
 
 	return 0;
@@ -156,10 +155,10 @@ static int log_append_stderr(const struct lxc_log_appender *appender,
 #endif
 
 	fprintf(stderr, "%s: %s%s", log_prefix,
-		log_container_name ? log_container_name : "",
-		log_container_name ? ": " : "");
+	        log_container_name ? log_container_name : "",
+	        log_container_name ? ": " : "");
 	fprintf(stderr, "%s: %s: %d ", event->locinfo->file,
-		event->locinfo->func, event->locinfo->line);
+	        event->locinfo->func, event->locinfo->line);
 	vfprintf(stderr, event->fmt, *event->vap);
 	fprintf(stderr, "\n");
 
@@ -167,7 +166,7 @@ static int log_append_stderr(const struct lxc_log_appender *appender,
 }
 
 /*---------------------------------------------------------------------------*/
-int lxc_unix_epoch_to_utc(char *buf, size_t bufsize, const struct timespec *time)
+static int lxc_unix_epoch_to_utc(char *buf, size_t bufsize, const struct timespec *time)
 {
 	int64_t epoch_to_days, z, era, doe, yoe, year, doy, mp, day, month,
 	    d_in_s, hours, h_in_s, minutes, seconds;
@@ -290,7 +289,8 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 {
 	char buffer[LXC_LOG_BUFFER_SIZE];
 	char date_time[LXC_LOG_TIME_SIZE];
-	int n, ret;
+	int n;
+	ssize_t ret;
 	int fd_to_use = -1;
 	const char *log_container_name = log_vmname;
 
@@ -314,16 +314,15 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 		return 0;
 
 	n = snprintf(buffer, sizeof(buffer),
-			"%s%s%s %s %-8s %s - %s:%s:%d - ",
-			log_prefix,
-			log_container_name ? " " : "",
-			log_container_name ? log_container_name : "",
-			date_time,
-			lxc_log_priority_to_string(event->priority),
-			event->category,
-			event->locinfo->file, event->locinfo->func,
-			event->locinfo->line);
-
+		     "%s%s%s %s %-8s %s - %s:%s:%d - ",
+		     log_prefix,
+		     log_container_name ? " " : "",
+		     log_container_name ? log_container_name : "",
+		     date_time,
+		     lxc_log_priority_to_string(event->priority),
+		     event->category,
+		     event->locinfo->file, event->locinfo->func,
+		     event->locinfo->line);
 	if (n < 0)
 		return n;
 
@@ -340,7 +339,12 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 
 	buffer[n] = '\n';
 
-	return write(fd_to_use, buffer, n + 1);
+again:
+	ret = write(fd_to_use, buffer, n + 1);
+	if (ret < 0 && errno == EINTR)
+		goto again;
+
+	return ret;
 }
 
 static struct lxc_log_appender log_appender_syslog = {
@@ -378,31 +382,31 @@ struct lxc_log_category lxc_log_category_lxc = {
 /*---------------------------------------------------------------------------*/
 static int build_dir(const char *name)
 {
-	int ret;
 	char *e, *n, *p;
 
-	/* Make copy of string since we'll be modifying it. */
+	/* Make copy of the string since we'll be modifying it. */
 	n = strdup(name);
-	if (!n) {
-		ERROR("Out of memory while creating directory '%s'.", name);
+	if (!n)
 		return -1;
-	}
 
 	e = &n[strlen(n)];
-	for (p = n+1; p < e; p++) {
+	for (p = n + 1; p < e; p++) {
+		int ret;
+
 		if (*p != '/')
 			continue;
 		*p = '\0';
-		if (access(n, F_OK)) {
-			ret = lxc_unpriv(mkdir(n, 0755));
-			if (ret && errno != EEXIST) {
-				SYSERROR("failed to create directory '%s'.", n);
-				free(n);
-				return -1;
-			}
+
+		ret = lxc_unpriv(mkdir(n, 0755));
+		if (ret && errno != EEXIST) {
+			SYSERROR("Failed to create directory %s", n);
+			free(n);
+			return -1;
 		}
+
 		*p = '/';
 	}
+
 	free(n);
 	return 0;
 }
@@ -413,20 +417,18 @@ static int log_open(const char *name)
 	int fd;
 	int newfd;
 
-	fd = lxc_unpriv(open(name, O_CREAT | O_WRONLY |
-			     O_APPEND | O_CLOEXEC, 0666));
-	if (fd == -1) {
-		ERROR("failed to open log file \"%s\" : %s", name,
-		      strerror(errno));
+	fd = lxc_unpriv(open(name, O_CREAT | O_WRONLY | O_APPEND | O_CLOEXEC, 0666));
+	if (fd < 0) {
+		SYSERROR("Failed to open log file \"%s\"", name);
 		return -1;
 	}
 
 	if (fd > 2)
 		return fd;
 
-	newfd = fcntl(fd, F_DUPFD_CLOEXEC, 3);
+	newfd = fcntl(fd, F_DUPFD_CLOEXEC, STDERR_FILENO);
 	if (newfd == -1)
-		ERROR("failed to dup log fd %d : %s", fd, strerror(errno));
+		SYSERROR("Failed to dup log fd %d", fd);
 
 	close(fd);
 	return newfd;
@@ -441,15 +443,17 @@ static int log_open(const char *name)
 static char *build_log_path(const char *name, const char *lxcpath)
 {
 	char *p;
-	int len, ret, use_dir;
+	int ret;
+	size_t len;
+	bool use_dir;
 
 	if (!name)
 		return NULL;
 
 #if USE_CONFIGPATH_LOGS
-	use_dir = 1;
+	use_dir = true;
 #else
-	use_dir = 0;
+	use_dir = false;
 #endif
 
 	/*
@@ -462,7 +466,7 @@ static char *build_log_path(const char *name, const char *lxcpath)
 	 */
 	len = strlen(name) + 6; /* 6 == '/' + '.log' + '\0' */
 	if (lxcpath)
-		use_dir = 1;
+		use_dir = true;
 	else
 		lxcpath = LOGPATH;
 
@@ -470,6 +474,7 @@ static char *build_log_path(const char *name, const char *lxcpath)
 		len += strlen(lxcpath) + 1 + strlen(name) + 1;  /* add "/$container_name/" */
 	else
 		len += strlen(lxcpath) + 1;
+
 	p = malloc(len);
 	if (!p)
 		return p;
@@ -478,25 +483,12 @@ static char *build_log_path(const char *name, const char *lxcpath)
 		ret = snprintf(p, len, "%s/%s/%s.log", lxcpath, name, name);
 	else
 		ret = snprintf(p, len, "%s/%s.log", lxcpath, name);
-
-	if (ret < 0 || ret >= len) {
+	if (ret < 0 || (size_t)ret >= len) {
 		free(p);
 		return NULL;
 	}
-	return p;
-}
 
-extern void lxc_log_close(void)
-{
-	closelog();
-	free(log_vmname);
-	log_vmname = NULL;
-	if (lxc_log_fd == -1)
-		return;
-	close(lxc_log_fd);
-	lxc_log_fd = -1;
-	free(log_fname);
-	log_fname = NULL;
+	return p;
 }
 
 /*
@@ -518,7 +510,7 @@ static int __lxc_log_set_file(const char *fname, int create_dirs)
 
 	if (strlen(fname) == 0) {
 		log_fname = NULL;
-		return 0;
+		return -1;
 	}
 
 #if USE_CONFIGPATH_LOGS
@@ -528,8 +520,7 @@ static int __lxc_log_set_file(const char *fname, int create_dirs)
 	if (create_dirs)
 #endif
 	if (build_dir(fname)) {
-		ERROR("failed to create dir for log file \"%s\" : %s", fname,
-		      strerror(errno));
+		SYSERROR("Failed to create dir for log file \"%s\"", fname);
 		return -1;
 	}
 
@@ -548,59 +539,30 @@ static int _lxc_log_set_file(const char *name, const char *lxcpath, int create_d
 
 	logfile = build_log_path(name, lxcpath);
 	if (!logfile) {
-		ERROR("could not build log path");
+		ERROR("Could not build log path");
 		return -1;
 	}
+
 	ret = __lxc_log_set_file(logfile, create_dirs);
 	free(logfile);
 	return ret;
 }
 
-extern int lxc_log_syslog(int facility)
-{
-	struct lxc_log_appender *appender;
-
-	openlog(log_prefix, LOG_PID, facility);
-	if (!lxc_log_category_lxc.appender) {
-		lxc_log_category_lxc.appender = &log_appender_syslog;
-		return 0;
-	}
-
-	appender = lxc_log_category_lxc.appender;
-	/* Check if syslog was already added, to avoid creating a loop */
-	while (appender) {
-		if (appender == &log_appender_syslog) {
-			/* not an error: openlog re-opened the connection */
-			return 0;
-		}
-		appender = appender->next;
-	}
-
-	appender = lxc_log_category_lxc.appender;
-	while (appender->next != NULL)
-		appender = appender->next;
-	appender->next = &log_appender_syslog;
-
-	return 0;
-}
-
-extern void lxc_log_enable_syslog(void)
-{
-	syslog_enable = 1;
-}
-
 /*
  * lxc_log_init:
  * Called from lxc front-end programs (like lxc-create, lxc-start) to
  * initalize the log defaults.
  */
-extern int lxc_log_init(struct lxc_log *log)
+int lxc_log_init(struct lxc_log *log)
 {
-	int lxc_priority = LXC_LOG_LEVEL_ERROR;
 	int ret;
+	int lxc_priority = LXC_LOG_LEVEL_ERROR;
+
+	if (!log)
+		return -1;
 
 	if (lxc_log_fd != -1) {
-		WARN("lxc_log_init called with log already initialized");
+		WARN("Log already initialized");
 		return 0;
 	}
 
@@ -612,10 +574,9 @@ extern int lxc_log_init(struct lxc_log *log)
 		lxc_loglevel_specified = 1;
 	}
 
-	if (!lxc_quiet_specified) {
+	if (!lxc_quiet_specified)
 		if (!log->quiet)
 			lxc_log_category_lxc.appender->next = &log_appender_stderr;
-	}
 
 	if (log->prefix)
 		lxc_log_set_prefix(log->prefix);
@@ -626,7 +587,13 @@ extern int lxc_log_init(struct lxc_log *log)
 	if (log->file) {
 		if (strcmp(log->file, "none") == 0)
 			return 0;
+
 		ret = __lxc_log_set_file(log->file, 1);
+		if (ret < 0) {
+			ERROR("Failed to enable logfile");
+			return -1;
+		}
+
 		lxc_log_use_global_fd = 1;
 	} else {
 		/* if no name was specified, there nothing to do */
@@ -656,38 +623,97 @@ extern int lxc_log_init(struct lxc_log *log)
 	 * ignore failures and continue logging to console
 	 */
 	if (!log->file && ret != 0) {
-		INFO("Ignoring failure to open default logfile.");
+		INFO("Ignoring failure to open default logfile");
 		ret = 0;
 	}
 
+	if (lxc_log_fd != -1) {
+		lxc_log_category_lxc.appender = &log_appender_logfile;
+		lxc_log_category_lxc.appender->next = &log_appender_stderr;
+	}
+
 	return ret;
 }
 
+void lxc_log_close(void)
+{
+	closelog();
+
+	free(log_vmname);
+	log_vmname = NULL;
+
+	if (lxc_log_fd == -1)
+		return;
+
+	close(lxc_log_fd);
+	lxc_log_fd = -1;
+
+	free(log_fname);
+	log_fname = NULL;
+}
+
+int lxc_log_syslog(int facility)
+{
+	struct lxc_log_appender *appender;
+
+	openlog(log_prefix, LOG_PID, facility);
+	if (!lxc_log_category_lxc.appender) {
+		lxc_log_category_lxc.appender = &log_appender_syslog;
+		return 0;
+	}
+
+	appender = lxc_log_category_lxc.appender;
+	/* Check if syslog was already added, to avoid creating a loop */
+	while (appender) {
+		if (appender == &log_appender_syslog) {
+			/* not an error: openlog re-opened the connection */
+			return 0;
+		}
+		appender = appender->next;
+	}
+
+	appender = lxc_log_category_lxc.appender;
+	while (appender->next != NULL)
+		appender = appender->next;
+	appender->next = &log_appender_syslog;
+
+	return 0;
+}
+
+inline void lxc_log_enable_syslog(void)
+{
+	syslog_enable = 1;
+}
+
 /*
  * This is called when we read a lxc.log.level entry in a lxc.conf file.  This
  * happens after processing command line arguments, which override the .conf
  * settings.  So only set the level if previously unset.
  */
-extern int lxc_log_set_level(int *dest, int level)
+int lxc_log_set_level(int *dest, int level)
 {
 	if (level < 0 || level >= LXC_LOG_LEVEL_NOTSET) {
-		ERROR("invalid log priority %d", level);
+		ERROR("Invalid log priority %d", level);
 		return -1;
 	}
+
 	*dest = level;
 	return 0;
 }
 
-extern int lxc_log_get_level(void)
+inline int lxc_log_get_level(void)
 {
 	return lxc_log_category_lxc.priority;
 }
 
-extern bool lxc_log_has_valid_level(void)
+bool lxc_log_has_valid_level(void)
 {
-	int log_level = lxc_log_get_level();
+	int log_level;
+
+	log_level = lxc_log_get_level();
 	if (log_level < 0 || log_level >= LXC_LOG_LEVEL_NOTSET)
 		return false;
+
 	return true;
 }
 
@@ -696,42 +722,40 @@ extern bool lxc_log_has_valid_level(void)
  * happens after processing command line arguments, which override the .conf
  * settings.  So only set the file if previously unset.
  */
-extern int lxc_log_set_file(int *fd, const char *fname)
+int lxc_log_set_file(int *fd, const char *fname)
 {
-	if (*fd != -1) {
+	if (*fd >= 0) {
 		close(*fd);
 		*fd = -1;
 	}
 
-	if (build_dir(fname)) {
-		ERROR("failed to create dir for log file \"%s\" : %s", fname,
-				strerror(errno));
+	if (build_dir(fname))
 		return -1;
-	}
 
 	*fd = log_open(fname);
-	if (*fd == -1)
-		return -errno;
+	if (*fd < 0)
+		return -1;
+
 	return 0;
 }
 
-extern const char *lxc_log_get_file(void)
+inline const char *lxc_log_get_file(void)
 {
 	return log_fname;
 }
 
-extern void lxc_log_set_prefix(const char *prefix)
+inline void lxc_log_set_prefix(const char *prefix)
 {
 	/* We don't care if thte prefix is truncated. */
 	(void)strlcpy(log_prefix, prefix, sizeof(log_prefix));
 }
 
-extern const char *lxc_log_get_prefix(void)
+inline const char *lxc_log_get_prefix(void)
 {
 	return log_prefix;
 }
 
-extern void lxc_log_options_no_override()
+inline void lxc_log_options_no_override()
 {
 	lxc_quiet_specified = 1;
 	lxc_loglevel_specified = 1;
