@@ -63,39 +63,7 @@
 #include "include/strlcpy.h"
 #endif
 
-#ifndef IFLA_LINKMODE
-#define IFLA_LINKMODE 17
-#endif
-
-#ifndef IFLA_LINKINFO
-#define IFLA_LINKINFO 18
-#endif
-
-#ifndef IFLA_NET_NS_PID
-#define IFLA_NET_NS_PID 19
-#endif
-
-#ifndef IFLA_INFO_KIND
-#define IFLA_INFO_KIND 1
-#endif
-
-#ifndef IFLA_VLAN_ID
-#define IFLA_VLAN_ID 1
-#endif
-
-#ifndef IFLA_INFO_DATA
-#define IFLA_INFO_DATA 2
-#endif
-
-#ifndef VETH_INFO_PEER
-#define VETH_INFO_PEER 1
-#endif
-
-#ifndef IFLA_MACVLAN_MODE
-#define IFLA_MACVLAN_MODE 1
-#endif
-
-lxc_log_define(lxc_network, lxc);
+lxc_log_define(network, lxc);
 
 typedef int (*instantiate_cb)(struct lxc_handler *, struct lxc_netdev *);
 
@@ -123,15 +91,18 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 		memcpy(netdev->priv.veth_attr.veth1, veth1, IFNAMSIZ);
 	}
 
-	snprintf(veth2buf, sizeof(veth2buf), "vethXXXXXX");
+	err = snprintf(veth2buf, sizeof(veth2buf), "vethXXXXXX");
+	if (err < 0 || (size_t)err >= sizeof(veth2buf))
+		return -1;
+
 	veth2 = lxc_mkifname(veth2buf);
 	if (!veth2)
 		goto out_delete;
 
 	err = lxc_veth_create(veth1, veth2);
 	if (err) {
-		ERROR("Failed to create veth pair \"%s\" and \"%s\": %s", veth1,
-		      veth2, strerror(-err));
+		errno = -err;
+		SYSERROR("Failed to create veth pair \"%s\" and \"%s\"", veth1, veth2);
 		goto out_delete;
 	}
 
@@ -140,8 +111,8 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 	 * of a container */
 	err = setup_private_host_hw_addr(veth1);
 	if (err) {
-		ERROR("Failed to change mac address of host interface \"%s\": %s",
-		      veth1, strerror(-err));
+		errno = -err;
+		SYSERROR("Failed to change mac address of host interface \"%s\"", veth1);
 		goto out_delete;
 	}
 
@@ -183,10 +154,11 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 		err = lxc_netdev_set_mtu(veth1, mtu);
 		if (!err)
 			err = lxc_netdev_set_mtu(veth2, mtu);
+
 		if (err) {
-			ERROR("Failed to set mtu \"%d\" for veth pair \"%s\" "
-			      "and \"%s\": %s",
-			      mtu, veth1, veth2, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to set mtu \"%d\" for veth pair \"%s\" "
+			         "and \"%s\"", mtu, veth1, veth2);
 			goto out_delete;
 		}
 	}
@@ -194,8 +166,9 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 	if (netdev->link[0] != '\0') {
 		err = lxc_bridge_attach(netdev->link, veth1);
 		if (err) {
-			ERROR("Failed to attach \"%s\" to bridge \"%s\": %s",
-			      veth1, netdev->link, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to attach \"%s\" to bridge \"%s\"",
+			         veth1, netdev->link);
 			goto out_delete;
 		}
 		INFO("Attached \"%s\" to bridge \"%s\"", veth1, netdev->link);
@@ -203,7 +176,8 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 
 	err = lxc_netdev_up(veth1);
 	if (err) {
-		ERROR("Failed to set \"%s\" up: %s", veth1, strerror(-err));
+		errno = -err;
+		SYSERROR("Failed to set \"%s\" up", veth1);
 		goto out_delete;
 	}
 
@@ -254,8 +228,9 @@ static int instantiate_macvlan(struct lxc_handler *handler, struct lxc_netdev *n
 	err = lxc_macvlan_create(netdev->link, peer,
 				 netdev->priv.macvlan_attr.mode);
 	if (err) {
-		ERROR("Failed to create macvlan interface \"%s\" on \"%s\": %s",
-		      peer, netdev->link, strerror(-err));
+		errno = -err;
+		SYSERROR("Failed to create macvlan interface \"%s\" on \"%s\"",
+		         peer, netdev->link);
 		goto on_error;
 	}
 
@@ -307,8 +282,9 @@ static int instantiate_vlan(struct lxc_handler *handler, struct lxc_netdev *netd
 
 	err = lxc_vlan_create(netdev->link, peer, netdev->priv.vlan_attr.vid);
 	if (err) {
-		ERROR("Failed to create vlan interface \"%s\" on \"%s\": %s",
-		      peer, netdev->link, strerror(-err));
+		errno = -err;
+		SYSERROR("Failed to create vlan interface \"%s\" on \"%s\"",
+		         peer, netdev->link);
 		return -1;
 	}
 
@@ -328,10 +304,12 @@ static int instantiate_vlan(struct lxc_handler *handler, struct lxc_netdev *netd
 			      netdev->name[0] != '\0' ? netdev->name : "(null)");
 			return -1;
 		}
+
 		err = lxc_netdev_set_mtu(peer, mtu);
 		if (err) {
-			ERROR("Failed to set mtu \"%s\" for \"%s\": %s",
-			      netdev->mtu, peer, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to set mtu \"%s\" for \"%s\"",
+			         netdev->mtu, peer);
 			lxc_netdev_delete_by_name(peer);
 			return -1;
 		}
@@ -1364,7 +1342,7 @@ static int proc_sys_net_write(const char *path, const char *value)
 	if (fd < 0)
 		return -errno;
 
-	if (write(fd, value, strlen(value)) < 0)
+	if (lxc_write_nointr(fd, value, strlen(value)) < 0)
 		err = -errno;
 
 	close(fd);
@@ -1955,12 +1933,18 @@ static const char padchar[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 char *lxc_mkifname(char *template)
 {
 	int ret;
-	unsigned int seed;
-	FILE *urandom;
 	struct ifaddrs *ifa, *ifaddr;
 	char name[IFNAMSIZ];
 	bool exists = false;
 	size_t i = 0;
+#ifdef HAVE_RAND_R
+	unsigned int seed;
+
+	seed = randseed(false);
+#else
+
+	(void)randseed(true);
+#endif
 
 	if (strlen(template) >= IFNAMSIZ)
 		return NULL;
@@ -1968,30 +1952,17 @@ char *lxc_mkifname(char *template)
 	/* Get all the network interfaces. */
 	ret = getifaddrs(&ifaddr);
 	if (ret < 0) {
-		ERROR("%s - Failed to get network interfaces", strerror(errno));
+		SYSERROR("Failed to get network interfaces");
 		return NULL;
 	}
 
-	/* Initialize the random number generator. */
-	urandom = fopen("/dev/urandom", "r");
-	if (urandom != NULL) {
-		if (fread(&seed, sizeof(seed), 1, urandom) <= 0)
-			seed = time(0);
-		fclose(urandom);
-	} else {
-		seed = time(0);
-	}
-
-#ifndef HAVE_RAND_R
-	srand(seed);
-#endif
-
 	/* Generate random names until we find one that doesn't exist. */
 	while (true) {
 		name[0] = '\0';
-		strcpy(name, template);
+		(void)strlcpy(name, template, IFNAMSIZ);
 
 		exists = false;
+
 		for (i = 0; i < strlen(name); i++) {
 			if (name[i] == 'X') {
 #ifdef HAVE_RAND_R
@@ -2014,7 +1985,9 @@ char *lxc_mkifname(char *template)
 	}
 
 	freeifaddrs(ifaddr);
-	return strcpy(template, name);
+	(void)strlcpy(template, name, strlen(template) + 1);
+
+	return template;
 }
 
 int setup_private_host_hw_addr(char *veth1)
@@ -2097,7 +2070,7 @@ int lxc_find_gateway_addresses(struct lxc_handler *handler)
 
 #define LXC_USERNIC_PATH LIBEXECDIR "/lxc/lxc-user-nic"
 static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcname,
-					  struct lxc_netdev *netdev, pid_t pid)
+					  struct lxc_netdev *netdev, pid_t pid, unsigned int hooks_version)
 {
 	int ret;
 	pid_t child;
@@ -2105,6 +2078,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcna
 	char *token, *saveptr = NULL;
 	char netdev_link[IFNAMSIZ];
 	char buffer[MAXPATHLEN] = {0};
+	size_t retlen;
 
 	if (netdev->type != LXC_NET_VETH) {
 		ERROR("Network type %d not support for unprivileged use", netdev->type);
@@ -2173,7 +2147,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcna
 	/* close the write-end of the pipe */
 	close(pipefd[1]);
 
-	bytes = read(pipefd[0], &buffer, MAXPATHLEN);
+	bytes = lxc_read_nointr(pipefd[0], &buffer, MAXPATHLEN);
 	if (bytes < 0) {
 		SYSERROR("Failed to read from pipe file descriptor");
 		close(pipefd[0]);
@@ -2209,8 +2183,8 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcna
 
 	ret = lxc_safe_int(token, &netdev->ifindex);
 	if (ret < 0) {
-		ERROR("%s - Failed to convert string \"%s\" to integer",
-		      strerror(-ret), token);
+		errno = -ret;
+		SYSERROR("Failed to convert string \"%s\" to integer", token);
 		return -1;
 	}
 
@@ -2221,12 +2195,12 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcna
 		return -1;
 	}
 
-	if (strlen(token) >= IFNAMSIZ) {
+	retlen = strlcpy(netdev->priv.veth_attr.veth1, token, IFNAMSIZ);
+	if (retlen >= IFNAMSIZ) {
 		ERROR("Host side veth device name returned by lxc-user-nic is "
 		      "too long");
 		return -E2BIG;
 	}
-	strcpy(netdev->priv.veth_attr.veth1, token);
 
 	/* netdev->priv.veth_attr.ifindex */
 	token = strtok_r(NULL, ":", &saveptr);
@@ -2237,11 +2211,26 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcna
 
 	ret = lxc_safe_int(token, &netdev->priv.veth_attr.ifindex);
 	if (ret < 0) {
-		ERROR("%s - Failed to convert string \"%s\" to integer",
-		      strerror(-ret), token);
+		errno = -ret;
+		SYSERROR("Failed to convert string \"%s\" to integer", token);
 		return -1;
 	}
 
+	if (netdev->upscript) {
+		char *argv[] = {
+			"veth",
+			netdev->link,
+			netdev->priv.veth_attr.veth1,
+			NULL,
+		};
+
+		ret = run_script_argv(lxcname,
+				hooks_version, "net",
+				netdev->upscript, "up", argv);
+		if (ret < 0)
+			return -1;
+    }
+
 	return 0;
 }
 
@@ -2314,7 +2303,7 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, const char *lxcna
 
 	close(pipefd[1]);
 
-	bytes = read(pipefd[0], &buffer, MAXPATHLEN);
+	bytes = lxc_read_nointr(pipefd[0], &buffer, MAXPATHLEN);
 	if (bytes < 0) {
 		SYSERROR("Failed to read from pipe file descriptor.");
 		close(pipefd[0]);
@@ -2483,9 +2472,9 @@ int lxc_network_move_created_netdev_priv(const char *lxcpath, const char *lxcnam
 
 		ret = lxc_netdev_move_by_name(ifname, pid, NULL);
 		if (ret) {
-			ERROR("Failed to move network device \"%s\" to "
-			      "network namespace %d: %s", ifname, pid,
-			      strerror(-ret));
+			errno = -ret;
+			SYSERROR("Failed to move network device \"%s\" to "
+			         "network namespace %d", ifname, pid);
 			return -1;
 		}
 
@@ -2499,7 +2488,7 @@ int lxc_network_move_created_netdev_priv(const char *lxcpath, const char *lxcnam
 }
 
 int lxc_create_network_unpriv(const char *lxcpath, const char *lxcname,
-			      struct lxc_list *network, pid_t pid)
+			      struct lxc_list *network, pid_t pid, unsigned int hooks_version)
 {
 	struct lxc_list *iterator;
 
@@ -2525,7 +2514,7 @@ int lxc_create_network_unpriv(const char *lxcpath, const char *lxcname,
 		if (netdev->mtu)
 			INFO("mtu ignored due to insufficient privilege");
 
-		if (lxc_create_network_unpriv_exec(lxcpath, lxcname, netdev, pid))
+		if (lxc_create_network_unpriv_exec(lxcpath, lxcname, netdev, pid, hooks_version))
 			return -1;
 	}
 
@@ -2578,10 +2567,10 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 			     netdev->name[0] != '\0' ? netdev->name : "(null)",
 			     netdev->ifindex);
 		} else if (ret < 0) {
-			WARN("Failed to remove interface \"%s\" with "
-			     "index %d: %s",
-			     netdev->name[0] != '\0' ? netdev->name : "(null)",
-			     netdev->ifindex, strerror(-ret));
+			errno = -ret;
+			SYSWARN("Failed to remove interface \"%s\" with index %d",
+			        netdev->name[0] != '\0' ? netdev->name : "(null)",
+			        netdev->ifindex);
 			goto clear_ifindices;
 		}
 		INFO("Removed interface \"%s\" with index %d",
@@ -2603,8 +2592,9 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 
 		ret = lxc_netdev_delete_by_name(hostveth);
 		if (ret < 0) {
-			WARN("Failed to remove interface \"%s\" from \"%s\": %s",
-			     hostveth, netdev->link, strerror(-ret));
+			errno = -ret;
+			SYSWARN("Failed to remove interface \"%s\" from \"%s\"",
+			        hostveth, netdev->link);
 			goto clear_ifindices;
 		}
 		INFO("Removed interface \"%s\" from \"%s\"", hostveth, netdev->link);
@@ -2734,12 +2724,12 @@ static int setup_hw_addr(char *hwaddr, const char *ifname)
 {
 	struct sockaddr sockaddr;
 	struct ifreq ifr;
-	int ret, fd, saved_errno;
+	int ret, fd;
 
 	ret = lxc_convert_mac(hwaddr, &sockaddr);
 	if (ret) {
-		ERROR("Mac address \"%s\" conversion failed: %s", hwaddr,
-		      strerror(-ret));
+		errno = -ret;
+		SYSERROR("Mac address \"%s\" conversion failed", hwaddr);
 		return -1;
 	}
 
@@ -2752,10 +2742,10 @@ static int setup_hw_addr(char *hwaddr, const char *ifname)
 		return -1;
 
 	ret = ioctl(fd, SIOCSIFHWADDR, &ifr);
-	saved_errno = errno;
-	close(fd);
 	if (ret)
-		ERROR("Failed to perform ioctl: %s", strerror(saved_errno));
+		SYSERROR("Failed to perform ioctl");
+
+	close(fd);
 
 	DEBUG("Mac address \"%s\" on \"%s\" has been setup", hwaddr,
 	      ifr.ifr_name);
@@ -2774,8 +2764,9 @@ static int setup_ipv4_addr(struct lxc_list *ip, int ifindex)
 		err = lxc_ipv4_addr_add(ifindex, &inetdev->addr,
 					&inetdev->bcast, inetdev->prefix);
 		if (err) {
-			ERROR("Failed to setup ipv4 address for network device "
-			      "with eifindex %d: %s", ifindex, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to setup ipv4 address for network device "
+			         "with eifindex %d", ifindex);
 			return -1;
 		}
 	}
@@ -2795,8 +2786,9 @@ static int setup_ipv6_addr(struct lxc_list *ip, int ifindex)
 					&inet6dev->mcast, &inet6dev->acast,
 					inet6dev->prefix);
 		if (err) {
-			ERROR("Failed to setup ipv6 address for network device "
-			      "with eifindex %d: %s", ifindex, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to setup ipv6 address for network device "
+			         "with eifindex %d", ifindex);
 			return -1;
 		}
 	}
@@ -2816,9 +2808,8 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 		if (netdev->flags & IFF_UP) {
 			err = lxc_netdev_up("lo");
 			if (err) {
-				ERROR("Failed to set the loopback network "
-				      "device up: %s",
-				      strerror(-err));
+				errno = -err;
+				SYSERROR("Failed to set the loopback network device up");
 				return -1;
 			}
 		}
@@ -2862,17 +2853,18 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 	 */
 	if (netdev->name[0] == '\0') {
 		if (netdev->type == LXC_NET_PHYS)
-			strcpy(netdev->name, netdev->link);
+			(void)strlcpy(netdev->name, netdev->link, IFNAMSIZ);
 		else
-			strcpy(netdev->name, "eth%d");
+			(void)strlcpy(netdev->name, "eth%d", IFNAMSIZ);
 	}
 
 	/* rename the interface name */
 	if (strcmp(ifname, netdev->name) != 0) {
 		err = lxc_netdev_rename_by_name(ifname, netdev->name);
 		if (err) {
-			ERROR("Failed to rename network device \"%s\" to "
-			      "\"%s\": %s", ifname, netdev->name, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to rename network device \"%s\" to \"%s\"",
+			         ifname, netdev->name);
 			return -1;
 		}
 	}
@@ -2890,7 +2882,7 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 	 * name of the network device in the child's network namespace. We will
 	 * later on send this information back to the parent.
 	 */
-	strcpy(netdev->name, current_ifname);
+	(void)strlcpy(netdev->name, current_ifname, IFNAMSIZ);
 
 	/* set a mac address */
 	if (netdev->hwaddr) {
@@ -2921,16 +2913,17 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 
 		err = lxc_netdev_up(current_ifname);
 		if (err) {
-			ERROR("Failed to set network device \"%s\" up: %s",
-			      current_ifname, strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to set network device \"%s\" up",
+			         current_ifname);
 			return -1;
 		}
 
 		/* the network is up, make the loopback up too */
 		err = lxc_netdev_up("lo");
 		if (err) {
-			ERROR("Failed to set the loopback network device up: %s",
-			      strerror(-err));
+			errno = -err;
+			SYSERROR("Failed to set the loopback network device up");
 			return -1;
 		}
 	}
@@ -2958,15 +2951,17 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 		if (err) {
 			err = lxc_ipv4_dest_add(netdev->ifindex, netdev->ipv4_gateway);
 			if (err) {
-				ERROR("Failed to add ipv4 dest for network "
-				      "device \"%s\": %s", ifname, strerror(-err));
+				errno = -err;
+				SYSERROR("Failed to add ipv4 dest for network device \"%s\"",
+				         ifname);
 			}
 
 			err = lxc_ipv4_gateway_add(netdev->ifindex, netdev->ipv4_gateway);
 			if (err) {
-				ERROR("Failed to setup ipv4 gateway for "
-				      "network device \"%s\": %s",
-				      ifname, strerror(-err));
+				errno = -err;
+				SYSERROR("Failed to setup ipv4 gateway for network device \"%s\"",
+				         ifname);
+
 				if (netdev->ipv4_gateway_auto) {
 					char buf[INET_ADDRSTRLEN];
 					inet_ntop(AF_INET, netdev->ipv4_gateway, buf, sizeof(buf));
@@ -2995,15 +2990,17 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 		if (err) {
 			err = lxc_ipv6_dest_add(netdev->ifindex, netdev->ipv6_gateway);
 			if (err) {
-				ERROR("Failed to add ipv6 dest for network "
-				      "device \"%s\": %s", ifname, strerror(-err));
+				errno = -err;
+				SYSERROR("Failed to add ipv6 dest for network device \"%s\"",
+				         ifname);
 			}
 
 			err = lxc_ipv6_gateway_add(netdev->ifindex, netdev->ipv6_gateway);
 			if (err) {
-				ERROR("Failed to setup ipv6 gateway for "
-				      "network device \"%s\": %s", ifname,
-				      strerror(-err));
+				errno = -err;
+				SYSERROR("Failed to setup ipv6 gateway for network device \"%s\"",
+				         ifname);
+
 				if (netdev->ipv6_gateway_auto) {
 					char buf[INET6_ADDRSTRLEN];
 					inet_ntop(AF_INET6, netdev->ipv6_gateway, buf, sizeof(buf));
@@ -3058,7 +3055,7 @@ int lxc_network_send_veth_names_to_child(struct lxc_handler *handler)
 		if (netdev->type != LXC_NET_VETH)
 			continue;
 
-		ret = send(data_sock, netdev->name, IFNAMSIZ, 0);
+		ret = send(data_sock, netdev->name, IFNAMSIZ, MSG_NOSIGNAL);
 		if (ret < 0)
 			return -1;
 		TRACE("Sent network device name \"%s\" to child", netdev->name);
@@ -3106,14 +3103,14 @@ int lxc_network_send_name_and_ifindex_to_parent(struct lxc_handler *handler)
 		struct lxc_netdev *netdev = iterator->elem;
 
 		/* Send network device name in the child's namespace to parent. */
-		ret = send(data_sock, netdev->name, IFNAMSIZ, 0);
+		ret = send(data_sock, netdev->name, IFNAMSIZ, MSG_NOSIGNAL);
 		if (ret < 0)
 			return -1;
 
 		/* Send network device ifindex in the child's namespace to
 		 * parent.
 		 */
-		ret = send(data_sock, &netdev->ifindex, sizeof(netdev->ifindex), 0);
+		ret = send(data_sock, &netdev->ifindex, sizeof(netdev->ifindex), MSG_NOSIGNAL);
 		if (ret < 0)
 			return -1;
 	}
@@ -3166,3 +3163,69 @@ void lxc_delete_network(struct lxc_handler *handler)
 	else
 		DEBUG("Deleted network devices");
 }
+
+int addattr(struct nlmsghdr *n, size_t maxlen, int type, const void *data, size_t alen)
+{
+	int len = RTA_LENGTH(alen);
+	struct rtattr *rta;
+
+	if (NLMSG_ALIGN(n->nlmsg_len) + RTA_ALIGN(len) > maxlen)
+		return -1;
+
+	rta = NLMSG_TAIL(n);
+	rta->rta_type = type;
+	rta->rta_len = len;
+	if (alen)
+		memcpy(RTA_DATA(rta), data, alen);
+	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + RTA_ALIGN(len);
+
+	return 0;
+}
+
+/* Attributes of RTM_NEWNSID/RTM_GETNSID messages */
+enum {
+	__LXC_NETNSA_NONE,
+#define __LXC_NETNSA_NSID_NOT_ASSIGNED -1
+	__LXC_NETNSA_NSID,
+	__LXC_NETNSA_PID,
+	__LXC_NETNSA_FD,
+	__LXC_NETNSA_MAX,
+};
+
+int lxc_netns_set_nsid(int fd)
+{
+	ssize_t ret;
+	char buf[NLMSG_ALIGN(sizeof(struct nlmsghdr)) +
+		 NLMSG_ALIGN(sizeof(struct rtgenmsg)) +
+		 NLMSG_ALIGN(1024)];
+	struct nl_handler nlh;
+	struct nlmsghdr *hdr;
+	struct rtgenmsg *msg;
+	__s32 ns_id = -1;
+	__u32 netns_fd = fd;
+
+	ret = netlink_open(&nlh, NETLINK_ROUTE);
+	if (ret < 0)
+		return ret;
+
+	memset(buf, 0, sizeof(buf));
+	hdr = (struct nlmsghdr *)buf;
+	msg = (struct rtgenmsg *)NLMSG_DATA(hdr);
+
+	hdr->nlmsg_len = NLMSG_LENGTH(sizeof(*msg));
+	hdr->nlmsg_type = RTM_NEWNSID;
+	hdr->nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK;
+	hdr->nlmsg_pid = 0;
+	hdr->nlmsg_seq = RTM_NEWNSID;
+	msg->rtgen_family = AF_UNSPEC;
+
+	addattr(hdr, 1024, __LXC_NETNSA_FD, &netns_fd, sizeof(netns_fd));
+	addattr(hdr, 1024, __LXC_NETNSA_NSID, &ns_id, sizeof(ns_id));
+
+	ret = __netlink_transaction(&nlh, hdr, hdr);
+	netlink_close(&nlh);
+	if (ret < 0)
+		return -1;
+
+	return 0;
+}
