@@ -61,7 +61,8 @@ static int nla_put(struct nlmsg *nlmsg, int attr,
 	rta = NLMSG_TAIL(nlmsg->nlmsghdr);
 	rta->rta_type = attr;
 	rta->rta_len = rtalen;
-	memcpy(RTA_DATA(rta), data, len);
+	if (data && len)
+		memcpy(RTA_DATA(rta), data, len);
 	nlmsg->nlmsghdr->nlmsg_len = tlen;
 	return 0;
 }
@@ -170,20 +171,20 @@ extern void nlmsg_free(struct nlmsg *nlmsg)
 	free(nlmsg);
 }
 
-extern int netlink_rcv(struct nl_handler *handler, struct nlmsg *answer)
+extern int __netlink_recv(struct nl_handler *handler, struct nlmsghdr *nlmsghdr)
 {
 	int ret;
 	struct sockaddr_nl nladdr;
 	struct iovec iov = {
-		.iov_base = answer->nlmsghdr,
-		.iov_len = answer->nlmsghdr->nlmsg_len,
+	    .iov_base = nlmsghdr,
+	    .iov_len = nlmsghdr->nlmsg_len,
 	};
 
 	struct msghdr msg = {
-		.msg_name = &nladdr,
-		.msg_namelen = sizeof(nladdr),
-		.msg_iov = &iov,
-		.msg_iovlen = 1,
+	    .msg_name = &nladdr,
+	    .msg_namelen = sizeof(nladdr),
+	    .msg_iov = &iov,
+	    .msg_iovlen = 1,
 	};
 
 	memset(&nladdr, 0, sizeof(nladdr));
@@ -196,70 +197,85 @@ again:
 	if (ret < 0) {
 		if (errno == EINTR)
 			goto again;
-		return -errno;
+
+		return -1;
 	}
 
 	if (!ret)
 		return 0;
 
-	if (msg.msg_flags & MSG_TRUNC &&
-	    ret == answer->nlmsghdr->nlmsg_len)
+	if (msg.msg_flags & MSG_TRUNC && (ret == nlmsghdr->nlmsg_len))
 		return -EMSGSIZE;
 
 	return ret;
 }
 
-extern int netlink_send(struct nl_handler *handler, struct nlmsg *nlmsg)
+extern int netlink_rcv(struct nl_handler *handler, struct nlmsg *answer)
 {
+	return __netlink_recv(handler, answer->nlmsghdr);
+}
+
+extern int __netlink_send(struct nl_handler *handler, struct nlmsghdr *nlmsghdr)
+{
+	int ret;
 	struct sockaddr_nl nladdr;
 	struct iovec iov = {
-		.iov_base = nlmsg->nlmsghdr,
-		.iov_len = nlmsg->nlmsghdr->nlmsg_len,
+	    .iov_base = nlmsghdr,
+	    .iov_len = nlmsghdr->nlmsg_len,
 	};
 	struct msghdr msg = {
-		.msg_name = &nladdr,
-		.msg_namelen = sizeof(nladdr),
-		.msg_iov = &iov,
-		.msg_iovlen = 1,
+	    .msg_name = &nladdr,
+	    .msg_namelen = sizeof(nladdr),
+	    .msg_iov = &iov,
+	    .msg_iovlen = 1,
 	};
-	int ret;
 
 	memset(&nladdr, 0, sizeof(nladdr));
 	nladdr.nl_family = AF_NETLINK;
 	nladdr.nl_pid = 0;
 	nladdr.nl_groups = 0;
 
-	ret = sendmsg(handler->fd, &msg, 0);
+	ret = sendmsg(handler->fd, &msg, MSG_NOSIGNAL);
 	if (ret < 0)
-		return -errno;
+		return -1;
 
 	return ret;
 }
 
-#ifndef NLMSG_ERROR
-#define NLMSG_ERROR                0x2
-#endif
-extern int netlink_transaction(struct nl_handler *handler,
-			       struct nlmsg *request, struct nlmsg *answer)
+extern int netlink_send(struct nl_handler *handler, struct nlmsg *nlmsg)
+{
+	return __netlink_send(handler, nlmsg->nlmsghdr);
+}
+
+extern int __netlink_transaction(struct nl_handler *handler,
+				 struct nlmsghdr *request,
+				 struct nlmsghdr *answer)
 {
 	int ret;
 
-	ret = netlink_send(handler, request);
+	ret = __netlink_send(handler, request);
 	if (ret < 0)
 		return ret;
 
-	ret = netlink_rcv(handler, answer);
+	ret = __netlink_recv(handler, answer);
 	if (ret < 0)
 		return ret;
 
-	if (answer->nlmsghdr->nlmsg_type == NLMSG_ERROR) {
-		struct nlmsgerr *err = (struct nlmsgerr*)NLMSG_DATA(answer->nlmsghdr);
+	if (answer->nlmsg_type == NLMSG_ERROR) {
+		struct nlmsgerr *err = (struct nlmsgerr *)NLMSG_DATA(answer);
 		return err->error;
 	}
 
 	return 0;
 }
 
+extern int netlink_transaction(struct nl_handler *handler,
+			       struct nlmsg *request, struct nlmsg *answer)
+{
+	return __netlink_transaction(handler, request->nlmsghdr,
+				     answer->nlmsghdr);
+}
+
 extern int netlink_open(struct nl_handler *handler, int protocol)
 {
 	socklen_t socklen;
