@@ -21,6 +21,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#define _GNU_SOURCE
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -42,8 +43,9 @@
 #include "lxc.h"
 #include "monitor.h"
 #include "start.h"
+#include "utils.h"
 
-lxc_log_define(lxc_state, lxc);
+lxc_log_define(state, lxc);
 
 static const char *const strstate[] = {
     "STOPPED",  "STARTING", "RUNNING", "STOPPING",
@@ -77,16 +79,15 @@ lxc_state_t lxc_getstate(const char *name, const char *lxcpath)
 
 static int fillwaitedstates(const char *strstates, lxc_state_t *states)
 {
-	char *token, *saveptr = NULL;
-	char *strstates_dup = strdup(strstates);
+	char *token;
+	char *strstates_dup;
 	int state;
 
+	strstates_dup = strdup(strstates);
 	if (!strstates_dup)
 		return -1;
 
-	token = strtok_r(strstates_dup, "|", &saveptr);
-	while (token) {
-
+	lxc_iterate_parts(token, strstates_dup, "|") {
 		state = lxc_str2state(token);
 		if (state < 0) {
 			free(strstates_dup);
@@ -94,23 +95,26 @@ static int fillwaitedstates(const char *strstates, lxc_state_t *states)
 		}
 
 		states[state] = 1;
-
-		token = strtok_r(NULL, "|", &saveptr);
 	}
 	free(strstates_dup);
 	return 0;
 }
 
-extern int lxc_wait(const char *lxcname, const char *states, int timeout,
-		    const char *lxcpath)
+int lxc_wait(const char *lxcname, const char *states, int timeout,
+	     const char *lxcpath)
 {
-	int state;
+	int state = -1;
 	lxc_state_t s[MAX_STATE] = {0};
 
 	if (fillwaitedstates(states, s))
 		return -1;
 
 	for (;;) {
+		struct timespec onesec = {
+		    .tv_sec = 1,
+		    .tv_nsec = 0,
+		};
+
 		state = lxc_cmd_sock_get_state(lxcname, lxcpath, s, timeout);
 		if (state >= 0)
 			break;
@@ -126,7 +130,12 @@ extern int lxc_wait(const char *lxcname, const char *states, int timeout,
 		if (timeout == 0)
 			return -1;
 
-		sleep(1);
+		(void)nanosleep(&onesec, NULL);
+	}
+
+	if (state < 0) {
+		ERROR("Failed to retrieve state from monitor");
+		return -1;
 	}
 
 	TRACE("Retrieved state of container %s", lxc_state2str(state));
