@@ -37,6 +37,10 @@
 #include "storage_utils.h"
 #include "utils.h"
 
+#ifndef HAVE_STRLCPY
+#include "include/strlcpy.h"
+#endif
+
 lxc_log_define(nbd, lxc);
 
 struct nbd_attach_data {
@@ -53,10 +57,14 @@ static bool wait_for_partition(const char *path);
 
 bool attach_nbd(char *src, struct lxc_conf *conf)
 {
-	char *orig = alloca(strlen(src)+1), *p, path[50];
+	char *orig, *p, path[50];
 	int i = 0;
+	size_t len;
+
+	len = strlen(src);
+	orig = alloca(len + 1);
+	(void)strlcpy(orig, src, len + 1);
 
-	strcpy(orig, src);
 	/* if path is followed by a partition, drop that for now */
 	p = strchr(orig, ':');
 	if (p)
@@ -201,7 +209,8 @@ static int do_attach_nbd(void *d)
 		exit(1);
 	}
 
-	if (prctl(PR_SET_PDEATHSIG, SIGHUP, 0, 0, 0) < 0)
+	if (prctl(PR_SET_PDEATHSIG, prctl_arg(SIGHUP), prctl_arg(0),
+		  prctl_arg(0), prctl_arg(0)) < 0)
 		SYSERROR("Error setting parent death signal for nbd watcher");
 
 	pid = fork();
