@@ -29,6 +29,7 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <lxc/lxccontainer.h>
 
 struct lxc_arguments;
@@ -93,7 +94,7 @@ struct lxc_arguments {
 	char *rbdname, *rbdpool;
 	char *zfsroot, *lowerdir, *dir;
 
-	/* lxc-execute */
+	/* lxc-execute and lxc-unshare */
 	uid_t uid;
 	gid_t gid;
 
@@ -136,6 +137,12 @@ struct lxc_arguments {
 	/* lxc-copy */
 	bool tmpfs;
 
+	/* lxc-unshare */
+	int flags;
+	int want_default_mounts;
+	const char *want_hostname;
+	bool setuid;
+
 	/* remaining arguments */
 	char *const *argv;
 	int argc;
@@ -146,7 +153,7 @@ struct lxc_arguments {
 
 #define LXC_COMMON_OPTIONS                                                     \
 	    { "name",        required_argument, 0, 'n'         },              \
-            { "help",        no_argument,       0, 'h'         },              \
+	    { "help",        no_argument,       0, 'h'         },              \
 	    { "usage",       no_argument,       0, OPT_USAGE   },              \
 	    { "version",     no_argument,       0, OPT_VERSION },              \
 	    { "quiet",       no_argument,       0, 'q'         },              \
@@ -171,10 +178,27 @@ extern int lxc_arguments_parse(struct lxc_arguments *args, int argc,
 extern int lxc_arguments_str_to_int(struct lxc_arguments *args,
 				    const char *str);
 
-#define lxc_error(arg, fmt, args...)                                           \
-	if (!(arg)->quiet)                                                     \
-	fprintf(stderr, "%s: " fmt "\n", (arg)->progname, ##args)
-
 extern bool lxc_setup_shared_ns(struct lxc_arguments *args, struct lxc_container *c);
 
+#define lxc_info(arg, fmt, args...)                                                \
+	do {                                                                       \
+		if (!(arg)->quiet) {                                               \
+			fprintf(stdout, "%s: " fmt "\n", (arg)->progname, ##args); \
+		}                                                                  \
+	} while (0)
+
+#define lxc_error(arg, fmt, args...)                                               \
+	do {                                                                       \
+		if (!(arg)->quiet) {                                               \
+			fprintf(stderr, "%s: " fmt "\n", (arg)->progname, ##args); \
+		}                                                                  \
+	} while (0)
+
+#define lxc_sys_error(arg, fmt, args...)                                                     \
+	do {                                                                                 \
+		if (!(arg)->quiet) {                                                         \
+			fprintf(stderr, "%s: " fmt "\n", (arg)->progname, ##args); \
+		}                                                                            \
+	} while (0)
+
 #endif /* __LXC_ARGUMENTS_H */
