@@ -51,6 +51,14 @@
 #include "parse.h"
 #include "utils.h"
 
+#ifndef HAVE_STRLCPY
+#include "include/strlcpy.h"
+#endif
+
+#ifndef HAVE_STRLCAT
+#include "include/strlcat.h"
+#endif
+
 #ifndef O_PATH
 #define O_PATH      010000000
 #endif
@@ -59,7 +67,7 @@
 #define O_NOFOLLOW  00400000
 #endif
 
-lxc_log_define(lxc_utils, lxc);
+lxc_log_define(utils, lxc);
 
 /*
  * if path is btrfs, tries to remove it and any subvolumes beneath it
@@ -85,9 +93,6 @@ static int _recursive_rmdir(const char *dirname, dev_t pdev,
 		struct stat mystat;
 		int rc;
 
-		if (!direntp)
-			break;
-
 		if (!strcmp(direntp->d_name, ".") ||
 		    !strcmp(direntp->d_name, ".."))
 			continue;
@@ -127,6 +132,7 @@ static int _recursive_rmdir(const char *dirname, dev_t pdev,
 			failed = 1;
 			continue;
 		}
+
 		if (onedev && mystat.st_dev != pdev) {
 			/* TODO should we be checking /proc/self/mountinfo for
 			 * pathname and not doing this if found? */
@@ -134,6 +140,7 @@ static int _recursive_rmdir(const char *dirname, dev_t pdev,
 				INFO("Removed btrfs subvolume at %s\n", pathname);
 			continue;
 		}
+
 		if (S_ISDIR(mystat.st_mode)) {
 			if (_recursive_rmdir(pathname, pdev, exclude, level+1, onedev) < 0)
 				failed=1;
@@ -228,6 +235,7 @@ extern int mkdir_p(const char *dir, mode_t mode)
 	do {
 		dir = tmp + strspn(tmp, "/");
 		tmp = dir + strcspn(dir, "/");
+
 		makeme = strndup(orig, dir - orig);
 		if (*makeme) {
 			if (mkdir(makeme, mode) && errno != EEXIST) {
@@ -248,9 +256,8 @@ char *get_rundir()
 	const char *homedir;
 	struct stat sb;
 
-	if (stat(RUNTIME_PATH, &sb) < 0) {
+	if (stat(RUNTIME_PATH, &sb) < 0)
 		return NULL;
-	}
 
 	if (geteuid() == sb.st_uid || getegid() == sb.st_gid) {
 		rundir = strdup(RUNTIME_PATH);
@@ -271,6 +278,9 @@ char *get_rundir()
 	}
 
 	rundir = malloc(sizeof(char) * (17 + strlen(homedir)));
+	if (!rundir)
+		return NULL;
+
 	sprintf(rundir, "%s/.cache/lxc/run/", homedir);
 
 	return rundir;
@@ -285,12 +295,16 @@ again:
 	if (ret == -1) {
 		if (errno == EINTR)
 			goto again;
+
 		return -1;
 	}
+
 	if (ret != pid)
 		goto again;
+
 	if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
 		return -1;
+
 	return 0;
 }
 
@@ -303,45 +317,54 @@ again:
 	if (ret == -1) {
 		if (errno == EINTR)
 			goto again;
+
 		return -1;
 	}
+
 	if (ret != pid)
 		goto again;
+
 	return status;
 }
 
-ssize_t lxc_write_nointr(int fd, const void* buf, size_t count)
+ssize_t lxc_write_nointr(int fd, const void *buf, size_t count)
 {
 	ssize_t ret;
 again:
 	ret = write(fd, buf, count);
 	if (ret < 0 && errno == EINTR)
 		goto again;
+
 	return ret;
 }
 
-ssize_t lxc_read_nointr(int fd, void* buf, size_t count)
+ssize_t lxc_read_nointr(int fd, void *buf, size_t count)
 {
 	ssize_t ret;
 again:
 	ret = read(fd, buf, count);
 	if (ret < 0 && errno == EINTR)
 		goto again;
+
 	return ret;
 }
 
-ssize_t lxc_read_nointr_expect(int fd, void* buf, size_t count, const void* expected_buf)
+ssize_t lxc_read_nointr_expect(int fd, void *buf, size_t count, const void *expected_buf)
 {
 	ssize_t ret;
+
 	ret = lxc_read_nointr(fd, buf, count);
 	if (ret <= 0)
 		return ret;
+
 	if ((size_t)ret != count)
 		return -1;
+
 	if (expected_buf && memcmp(buf, expected_buf, count) != 0) {
 		errno = EINVAL;
 		return -1;
 	}
+
 	return ret;
 }
 
@@ -364,42 +387,50 @@ int sha1sum_file(char *fnam, unsigned char *digest)
 
 	if (!fnam)
 		return -1;
+
 	f = fopen_cloexec(fnam, "r");
 	if (!f) {
 		SYSERROR("Error opening template");
 		return -1;
 	}
+
 	if (fseek(f, 0, SEEK_END) < 0) {
 		SYSERROR("Error seeking to end of template");
 		fclose(f);
 		return -1;
 	}
+
 	if ((flen = ftell(f)) < 0) {
 		SYSERROR("Error telling size of template");
 		fclose(f);
 		return -1;
 	}
+
 	if (fseek(f, 0, SEEK_SET) < 0) {
 		SYSERROR("Error seeking to start of template");
 		fclose(f);
 		return -1;
 	}
+
 	if ((buf = malloc(flen+1)) == NULL) {
 		SYSERROR("Out of memory");
 		fclose(f);
 		return -1;
 	}
+
 	if (fread(buf, 1, flen, f) != flen) {
 		SYSERROR("Failure reading template");
 		free(buf);
 		fclose(f);
 		return -1;
 	}
+
 	if (fclose(f) < 0) {
 		SYSERROR("Failre closing template");
 		free(buf);
 		return -1;
 	}
+
 	buf[flen] = '\0';
 	ret = gnutls_hash_fast(GNUTLS_DIG_SHA1, buf, flen, (void *)digest);
 	free(buf);
@@ -428,6 +459,7 @@ char** lxc_va_arg_list_to_argv(va_list ap, size_t skip, int do_strdup)
 	result = calloc(count, sizeof(char*));
 	if (!result)
 		return NULL;
+
 	count = skip;
 	while (1) {
 		char* arg = va_arg(ap, char*);
@@ -510,6 +542,7 @@ struct lxc_popen_FILE *lxc_popen(const char *command)
 	fp = malloc(sizeof(*fp));
 	if (!fp)
 		goto on_error;
+
 	memset(fp, 0, sizeof(*fp));
 
 	fp->child_pid = child_pid;
@@ -582,6 +615,7 @@ char *lxc_string_replace(const char *needle, const char *replacement, const char
 			result = calloc(1, len + 1);
 			if (!result)
 				return NULL;
+
 			saved_len = len;
 		}
 
@@ -591,15 +625,20 @@ char *lxc_string_replace(const char *needle, const char *replacement, const char
 			part_len = (ssize_t)(p - last_p);
 			if (result && part_len > 0)
 				memcpy(&result[len], last_p, part_len);
+
 			len += part_len;
+
 			if (result && replacement_len > 0)
 				memcpy(&result[len], replacement, replacement_len);
+
 			len += replacement_len;
 			p += needle_len;
 		}
+
 		part_len = strlen(last_p);
 		if (result && part_len > 0)
 			memcpy(&result[len], last_p, part_len);
+
 		len += part_len;
 	}
 
@@ -610,6 +649,7 @@ char *lxc_string_replace(const char *needle, const char *replacement, const char
 		free(result);
 		return NULL;
 	}
+
 	/* make sure we didn't overwrite any buffer,
 	 * due to calloc the string should be 0-terminated */
 	if (result[len] != '\0') {
@@ -625,6 +665,7 @@ bool lxc_string_in_array(const char *needle, const char **haystack)
 	for (; haystack && *haystack; haystack++)
 		if (!strcmp(needle, *haystack))
 			return true;
+
 	return false;
 }
 
@@ -634,21 +675,25 @@ char *lxc_string_join(const char *sep, const char **parts, bool use_as_prefix)
 	char **p;
 	size_t sep_len = strlen(sep);
 	size_t result_len = use_as_prefix * sep_len;
+	size_t buf_len;
 
 	/* calculate new string length */
 	for (p = (char **)parts; *p; p++)
 		result_len += (p > (char **)parts) * sep_len + strlen(*p);
 
-	result = calloc(result_len + 1, 1);
+	buf_len = result_len + 1;
+	result = calloc(buf_len, 1);
 	if (!result)
 		return NULL;
 
 	if (use_as_prefix)
-		strcpy(result, sep);
+		(void)strlcpy(result, sep, buf_len);
+
 	for (p = (char **)parts; *p; p++) {
 		if (p > (char **)parts)
-			strcat(result, sep);
-		strcat(result, *p);
+			(void)strlcat(result, sep, buf_len);
+
+		(void)strlcat(result, *p, buf_len);
 	}
 
 	return result;
@@ -664,6 +709,7 @@ char **lxc_normalize_path(const char *path)
 	components = lxc_string_split(path, '/');
 	if (!components)
 		return NULL;
+
 	for (p = components; *p; p++)
 		components_len++;
 
@@ -712,6 +758,7 @@ char *lxc_deslashify(const char *path)
 			lxc_free_array((void **)parts, free);
 			return dup;
 		}
+
 		n = strcspn(dup, "/");
 		if (n == len) {
 			free(dup);
@@ -759,43 +806,50 @@ char *lxc_append_paths(const char *first, const char *second)
 
 bool lxc_string_in_list(const char *needle, const char *haystack, char _sep)
 {
-	char *token, *str, *saveptr = NULL;
+	char *token, *str;
 	char sep[2] = { _sep, '\0' };
+	size_t len;
 
 	if (!haystack || !needle)
 		return 0;
 
-	str = alloca(strlen(haystack)+1);
-	strcpy(str, haystack);
-	for (; (token = strtok_r(str, sep, &saveptr)); str = NULL) {
+	len = strlen(haystack);
+	str = alloca(len + 1);
+	(void)strlcpy(str, haystack, len + 1);
+
+	lxc_iterate_parts(token, str, sep)
 		if (strcmp(needle, token) == 0)
 			return 1;
-	}
 
 	return 0;
 }
 
 char **lxc_string_split(const char *string, char _sep)
 {
-	char *token, *str, *saveptr = NULL;
+	char *token, *str;
 	char sep[2] = {_sep, '\0'};
 	char **tmp = NULL, **result = NULL;
 	size_t result_capacity = 0;
 	size_t result_count = 0;
 	int r, saved_errno;
+	size_t len;
 
 	if (!string)
 		return calloc(1, sizeof(char *));
 
-	str = alloca(strlen(string) + 1);
-	strcpy(str, string);
-	for (; (token = strtok_r(str, sep, &saveptr)); str = NULL) {
+	len = strlen(string);
+	str = alloca(len + 1);
+	(void)strlcpy(str, string, len + 1);
+
+	lxc_iterate_parts(token, str, sep) {
 		r = lxc_grow_array((void ***)&result, &result_capacity, result_count + 1, 16);
 		if (r < 0)
 			goto error_out;
+
 		result[result_count] = strdup(token);
 		if (!result[result_count])
 			goto error_out;
+
 		result_count++;
 	}
 
@@ -803,11 +857,15 @@ char **lxc_string_split(const char *string, char _sep)
 	tmp = realloc(result, (result_count + 1) * sizeof(char *));
 	if (!tmp)
 		goto error_out;
+
 	result = tmp;
+
 	/* Make sure we don't return uninitialized memory. */
 	if (result_count == 0)
 		*result = NULL;
+
 	return result;
+
 error_out:
 	saved_errno = errno;
 	lxc_free_array((void **)result, free);
@@ -822,9 +880,11 @@ static bool complete_word(char ***result, char *start, char *end, size_t *cap, s
 	r = lxc_grow_array((void ***)result, cap, 2 + *cnt, 16);
 	if (r < 0)
 		return false;
+
 	(*result)[*cnt] = strndup(start, end - start);
 	if (!(*result)[*cnt])
 		return false;
+
 	(*cnt)++;
 
 	return true;
@@ -885,38 +945,46 @@ char **lxc_string_split_quoted(char *string)
 
 char **lxc_string_split_and_trim(const char *string, char _sep)
 {
-	char *token, *str, *saveptr = NULL;
+	char *token, *str;
 	char sep[2] = { _sep, '\0' };
 	char **result = NULL;
 	size_t result_capacity = 0;
 	size_t result_count = 0;
 	int r, saved_errno;
 	size_t i = 0;
+	size_t len;
 
 	if (!string)
 		return calloc(1, sizeof(char *));
 
-	str = alloca(strlen(string)+1);
-	strcpy(str, string);
-	for (; (token = strtok_r(str, sep, &saveptr)); str = NULL) {
+	len = strlen(string);
+	str = alloca(len + 1);
+	(void)strlcpy(str, string, len + 1);
+
+	lxc_iterate_parts(token, str, sep) {
 		while (token[0] == ' ' || token[0] == '\t')
 			token++;
+
 		i = strlen(token);
 		while (i > 0 && (token[i - 1] == ' ' || token[i - 1] == '\t')) {
 			token[i - 1] = '\0';
 			i--;
 		}
+
 		r = lxc_grow_array((void ***)&result, &result_capacity, result_count + 1, 16);
 		if (r < 0)
 			goto error_out;
+
 		result[result_count] = strdup(token);
 		if (!result[result_count])
 			goto error_out;
+
 		result_count++;
 	}
 
 	/* if we allocated too much, reduce it */
 	return realloc(result, (result_count + 1) * sizeof(char *));
+
 error_out:
 	saved_errno = errno;
 	lxc_free_array((void **)result, free);
@@ -927,12 +995,14 @@ error_out:
 void lxc_free_array(void **array, lxc_free_fn element_free_fn)
 {
 	void **p;
+
 	for (p = array; p && *p; p++)
 		element_free_fn(*p);
+
 	free((void*)array);
 }
 
-int lxc_grow_array(void ***array, size_t* capacity, size_t new_size, size_t capacity_increment)
+int lxc_grow_array(void ***array, size_t *capacity, size_t new_size, size_t capacity_increment)
 {
 	size_t new_capacity;
 	void **new_array;
@@ -947,11 +1017,13 @@ int lxc_grow_array(void ***array, size_t* capacity, size_t new_size, size_t capa
 	new_capacity = *capacity;
 	while (new_size + 1 > new_capacity)
 		new_capacity += capacity_increment;
+
 	if (new_capacity != *capacity) {
 		/* we have to reallocate */
 		new_array = realloc(*array, new_capacity * sizeof(void *));
 		if (!new_array)
 			return -1;
+
 		memset(&new_array[*capacity], 0, (new_capacity - (*capacity)) * sizeof(void *));
 		*array = new_array;
 		*capacity = new_capacity;
@@ -981,16 +1053,20 @@ int lxc_write_to_file(const char *filename, const void *buf, size_t count,
 	fd = open(filename, O_WRONLY | O_TRUNC | O_CREAT | O_CLOEXEC, mode);
 	if (fd < 0)
 		return -1;
+
 	ret = lxc_write_nointr(fd, buf, count);
 	if (ret < 0)
 		goto out_error;
+
 	if ((size_t)ret != count)
 		goto out_error;
+
 	if (add_newline) {
 		ret = lxc_write_nointr(fd, "\n", 1);
 		if (ret != 1)
 			goto out_error;
 	}
+
 	close(fd);
 	return 0;
 
@@ -1001,7 +1077,7 @@ out_error:
 	return -1;
 }
 
-int lxc_read_from_file(const char *filename, void* buf, size_t count)
+int lxc_read_from_file(const char *filename, void *buf, size_t count)
 {
 	int fd = -1, saved_errno;
 	ssize_t ret;
@@ -1013,17 +1089,19 @@ int lxc_read_from_file(const char *filename, void* buf, size_t count)
 	if (!buf || !count) {
 		char buf2[100];
 		size_t count2 = 0;
-		while ((ret = read(fd, buf2, 100)) > 0)
+
+		while ((ret = lxc_read_nointr(fd, buf2, 100)) > 0)
 			count2 += ret;
+
 		if (ret >= 0)
 			ret = count2;
 	} else {
 		memset(buf, 0, count);
-		ret = read(fd, buf, count);
+		ret = lxc_read_nointr(fd, buf, count);
 	}
 
 	if (ret < 0)
-		ERROR("read %s: %s", filename, strerror(errno));
+		SYSERROR("Read %s", filename);
 
 	saved_errno = errno;
 	close(fd);
@@ -1045,9 +1123,11 @@ void **lxc_append_null_to_array(void **array, size_t count)
 			free(array);
 			return NULL;
 		}
+
 		array = temp;
 		array[count] = NULL;
 	}
+
 	return array;
 }
 
@@ -1063,7 +1143,8 @@ int randseed(bool srand_it)
 	if (f) {
 		int ret = fread(&seed, sizeof(seed), 1, f);
 		if (ret != 1)
-			DEBUG("unable to fread /dev/urandom, %s, fallback to time+pid rand seed", strerror(errno));
+			SYSDEBUG("unable to fread /dev/urandom, fallback to time+pid rand seed");
+
 		fclose(f);
 	}
 
@@ -1085,6 +1166,7 @@ uid_t get_ns_uid(uid_t orig)
 	while (getline(&line, &sz, f) != -1) {
 		if (sscanf(line, "%u %u %u", &nsid, &hostid, &range) != 3)
 			continue;
+
 		if (hostid <= orig && hostid + range > orig) {
 			nsid += orig - hostid;
 			goto found;
@@ -1092,6 +1174,7 @@ uid_t get_ns_uid(uid_t orig)
 	}
 
 	nsid = 0;
+
 found:
 	fclose(f);
 	free(line);
@@ -1107,6 +1190,7 @@ bool dir_exists(const char *path)
 	if (ret < 0)
 		/* Could be something other than eexist, just say "no". */
 		return false;
+
 	return S_ISDIR(sb.st_mode);
 }
 
@@ -1151,14 +1235,17 @@ int detect_shared_rootfs(void)
 	f = fopen("/proc/self/mountinfo", "r");
 	if (!f)
 		return 0;
+
 	while (fgets(buf, LXC_LINELEN, f)) {
 		for (p = buf, i = 0; p && i < 4; i++)
 			p = strchr(p + 1, ' ');
 		if (!p)
 			continue;
+
 		p2 = strchr(p + 1, ' ');
 		if (!p2)
 			continue;
+
 		*p2 = '\0';
 		if (strcmp(p + 1, "/") == 0) {
 			/* This is '/'. Is it shared? */
@@ -1169,6 +1256,7 @@ int detect_shared_rootfs(void)
 			}
 		}
 	}
+
 	fclose(f);
 	return 0;
 }
@@ -1184,16 +1272,17 @@ bool switch_to_ns(pid_t pid, const char *ns) {
 
 	fd = open(nspath, O_RDONLY);
 	if (fd < 0) {
-		SYSERROR("failed to open %s", nspath);
+		SYSERROR("Failed to open %s", nspath);
 		return false;
 	}
 
 	ret = setns(fd, 0);
 	if (ret) {
-		SYSERROR("failed to set process %d to %s of %d.", pid, ns, fd);
+		SYSERROR("Failed to set process %d to %s of %d.", pid, ns, fd);
 		close(fd);
 		return false;
 	}
+
 	close(fd);
 	return true;
 }
@@ -1222,9 +1311,11 @@ bool detect_ramfs_rootfs(void)
 			p = strchr(p + 1, ' ');
 		if (!p)
 			continue;
+
 		p2 = strchr(p + 1, ' ');
 		if (!p2)
 			continue;
+
 		*p2 = '\0';
 		if (strcmp(p + 1, "/") == 0) {
 			/* This is '/'. Is it the ramfs? */
@@ -1232,19 +1323,19 @@ bool detect_ramfs_rootfs(void)
 			if (p && strncmp(p, "- rootfs rootfs ", 16) == 0) {
 				free(line);
 				fclose(f);
+				INFO("Rootfs is located on ramfs");
 				return true;
 			}
 		}
 	}
+
 	free(line);
 	fclose(f);
 	return false;
 }
 
 char *on_path(const char *cmd, const char *rootfs) {
-	char *path = NULL;
-	char *entry = NULL;
-	char *saveptr = NULL;
+	char *entry = NULL, *path = NULL;
 	char cmdpath[MAXPATHLEN];
 	int ret;
 
@@ -1256,23 +1347,18 @@ char *on_path(const char *cmd, const char *rootfs) {
 	if (!path)
 		return NULL;
 
-	entry = strtok_r(path, ":", &saveptr);
-	while (entry) {
+	lxc_iterate_parts(entry, path, ":") {
 		if (rootfs)
 			ret = snprintf(cmdpath, MAXPATHLEN, "%s/%s/%s", rootfs, entry, cmd);
 		else
 			ret = snprintf(cmdpath, MAXPATHLEN, "%s/%s", entry, cmd);
-
 		if (ret < 0 || ret >= MAXPATHLEN)
-			goto next_loop;
+			continue;
 
 		if (access(cmdpath, X_OK) == 0) {
 			free(path);
 			return strdup(cmdpath);
 		}
-
-next_loop:
-		entry = strtok_r(NULL, ":", &saveptr);
 	}
 
 	free(path);
@@ -1304,6 +1390,7 @@ char *choose_init(const char *rootfs)
 	if (!getenv("PATH")) {
 		if (setenv("PATH", "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin", 0))
 			SYSERROR("Failed to setenv");
+
 		env_set = 1;
 	}
 
@@ -1331,6 +1418,7 @@ char *choose_init(const char *rootfs)
 		ERROR("pathname too long");
 		goto out1;
 	}
+
 	if (access(retv, X_OK) == 0)
 		return retv;
 
@@ -1339,6 +1427,7 @@ char *choose_init(const char *rootfs)
 		ERROR("pathname too long");
 		goto out1;
 	}
+
 	if (access(retv, X_OK) == 0)
 		return retv;
 
@@ -1347,6 +1436,7 @@ char *choose_init(const char *rootfs)
 		ERROR("pathname too long");
 		goto out1;
 	}
+
 	if (access(retv, X_OK) == 0)
 		return retv;
 
@@ -1355,6 +1445,7 @@ char *choose_init(const char *rootfs)
 		ERROR("pathname too long");
 		goto out1;
 	}
+
 	if (access(retv, X_OK) == 0)
 		return retv;
 
@@ -1373,6 +1464,7 @@ char *choose_init(const char *rootfs)
 		WARN("Nonsense - name /lxc.init.static too long");
 		goto out1;
 	}
+
 	if (access(retv, X_OK) == 0)
 		return retv;
 
@@ -1389,8 +1481,10 @@ int print_to_file(const char *file, const char *content)
 	f = fopen(file, "w");
 	if (!f)
 		return -1;
+
 	if (fprintf(f, "%s", content) != strlen(content))
 		ret = -1;
+
 	fclose(f);
 	return ret;
 }
@@ -1398,9 +1492,12 @@ int print_to_file(const char *file, const char *content)
 int is_dir(const char *path)
 {
 	struct stat statbuf;
-	int ret = stat(path, &statbuf);
+	int ret;
+
+	ret = stat(path, &statbuf);
 	if (ret == 0 && S_ISDIR(statbuf.st_mode))
 		return 1;
+
 	return 0;
 }
 
@@ -1421,14 +1518,17 @@ char *get_template_path(const char *t)
 	}
 
 	len = strlen(LXCTEMPLATEDIR) + strlen(t) + strlen("/lxc-") + 1;
+
 	tpath = malloc(len);
 	if (!tpath)
 		return NULL;
+
 	ret = snprintf(tpath, len, "%s/lxc-%s", LXCTEMPLATEDIR, t);
 	if (ret < 0 || ret >= len) {
 		free(tpath);
 		return NULL;
 	}
+
 	if (access(tpath, X_OK) < 0) {
 		SYSERROR("bad template: %s", t);
 		free(tpath);
@@ -1454,6 +1554,7 @@ static char *get_nextpath(char *path, int *offsetp, int fulllen)
 
 	while (path[offset] != '\0' && offset < fulllen)
 		offset++;
+
 	while (path[offset] == '\0' && offset < fulllen)
 		offset++;
 
@@ -1471,12 +1572,16 @@ static bool is_subdir(const char *subdir, const char *dir, size_t len)
 
 	if (subdirlen < len)
 		return false;
+
 	if (strncmp(subdir, dir, len) != 0)
 		return false;
+
 	if (dir[len-1] == '/')
 		return true;
+
 	if (subdir[len] == '/' || subdirlen == len)
 		return true;
+
 	return false;
 }
 
@@ -1487,11 +1592,15 @@ static bool is_subdir(const char *subdir, const char *dir, size_t len)
 static int check_symlink(int fd)
 {
 	struct stat sb;
-	int ret = fstat(fd, &sb);
+	int ret;
+
+	ret = fstat(fd, &sb);
 	if (ret < 0)
 		return -ENOENT;
+
 	if (S_ISLNK(sb.st_mode))
 		return -ELOOP;
+
 	return 0;
 }
 
@@ -1559,6 +1668,7 @@ static int open_without_symlink(const char *target, const char *prefix_skip)
 				target, prefix_skip);
 			return -EINVAL;
 		}
+
 		/*
 		 * get_nextpath() expects the curlen argument to be
 		 * on a  (turned into \0) / or before it, so decrement
@@ -1576,6 +1686,7 @@ static int open_without_symlink(const char *target, const char *prefix_skip)
 		SYSERROR("Out of memory checking for symbolic link");
 		return -ENOMEM;
 	}
+
 	for (i = 0; i < fulllen; i++) {
 		if (dup[i] == '/')
 			dup[i] = '\0';
@@ -1584,20 +1695,24 @@ static int open_without_symlink(const char *target, const char *prefix_skip)
 	dirfd = open(prefix_skip, O_RDONLY);
 	if (dirfd < 0)
 		goto out;
+
 	while (1) {
 		int newfd, saved_errno;
 		char *nextpath;
 
 		if ((nextpath = get_nextpath(dup, &curlen, fulllen)) == NULL)
 			goto out;
+
 		newfd = open_if_safe(dirfd, nextpath);
 		saved_errno = errno;
 		close(dirfd);
+
 		dirfd = newfd;
 		if (newfd < 0) {
 			errno = saved_errno;
 			if (errno == ELOOP)
 				SYSERROR("%s in %s was a symbolic link!", nextpath, target);
+
 			goto out;
 		}
 	}
@@ -1630,9 +1745,11 @@ int safe_mount(const char *src, const char *dest, const char *fstype,
 	/* todo - allow symlinks for relative paths if 'allowsymlinks' option is passed */
 	if (flags & MS_BIND && src && src[0] != '/') {
 		INFO("this is a relative bind mount");
+
 		srcfd = open_without_symlink(src, NULL);
 		if (srcfd < 0)
 			return srcfd;
+
 		ret = snprintf(srcbuf, 50, "/proc/self/fd/%d", srcfd);
 		if (ret < 0 || ret > 50) {
 			close(srcfd);
@@ -1649,6 +1766,7 @@ int safe_mount(const char *src, const char *dest, const char *fstype,
 			close(srcfd);
 			errno = saved_errno;
 		}
+
 		return destfd;
 	}
 
@@ -1656,6 +1774,7 @@ int safe_mount(const char *src, const char *dest, const char *fstype,
 	if (ret < 0 || ret > 50) {
 		if (srcfd != -1)
 			close(srcfd);
+
 		close(destfd);
 		ERROR("Out of memory");
 		return -EINVAL;
@@ -1665,6 +1784,7 @@ int safe_mount(const char *src, const char *dest, const char *fstype,
 	saved_errno = errno;
 	if (srcfd != -1)
 		close(srcfd);
+
 	close(destfd);
 	if (ret < 0) {
 		errno = saved_errno;
@@ -1711,6 +1831,7 @@ int lxc_mount_proc_if_needed(const char *rootfs)
 	if (linklen < 0) {
 		if (mkdir(path, 0755) && errno != EEXIST)
 			return -1;
+
 		goto domount;
 	} else if (linklen >= LXC_NUMSTRLEN64) {
 		link[linklen - 1] = '\0';
@@ -1780,8 +1901,9 @@ int set_stdfds(int fd)
 int null_stdfds(void)
 {
 	int ret = -1;
-	int fd = open_devnull();
+	int fd;
 
+	fd = open_devnull();
 	if (fd >= 0) {
 		ret = set_stdfds(fd);
 		close(fd);
@@ -1807,6 +1929,7 @@ int lxc_count_file_lines(const char *fn)
 	while (getline(&line, &sz, f) != -1) {
 		n++;
 	}
+
 	free(line);
 	fclose(f);
 	return n;
@@ -1908,9 +2031,10 @@ int lxc_preserve_ns(const int pid, const char *ns)
 	ret = snprintf(path, __NS_PATH_LEN, "/proc/%d/ns%s%s", pid,
 		       !ns || strcmp(ns, "") == 0 ? "" : "/",
 		       !ns || strcmp(ns, "") == 0 ? "" : ns);
-	errno = EFBIG;
-	if (ret < 0 || (size_t)ret >= __NS_PATH_LEN)
-		return -EFBIG;
+	if (ret < 0 || (size_t)ret >= __NS_PATH_LEN) {
+		errno = EFBIG;
+		return -1;
+	}
 
 	return open(path, O_RDONLY | O_CLOEXEC);
 }
@@ -2091,9 +2215,6 @@ static int lxc_get_unused_loop_dev_legacy(char *loop_name)
 		return -1;
 
 	while ((dp = readdir(dir))) {
-		if (!dp)
-			break;
-
 		if (strncmp(dp->d_name, "loop", 4) != 0)
 			continue;
 
@@ -2224,6 +2345,7 @@ pop_stack:
 		 */
 		if (umounts != INT_MAX)
 			umounts++;
+
 		/* We succeeded in umounting. Make sure that there's no other
 		 * mountpoint stacked underneath.
 		 */
@@ -2239,7 +2361,7 @@ int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 	int ret, fret, pipefd[2];
 	ssize_t bytes;
 
-	/* Make sure our callers do not receive unitialized memory. */
+	/* Make sure our callers do not receive uninitialized memory. */
 	if (buf_size > 0 && buf)
 		buf[0] = '\0';
 
@@ -2285,7 +2407,7 @@ int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 	close(pipefd[1]);
 
 	if (buf && buf_size > 0) {
-		bytes = read(pipefd[0], buf, buf_size - 1);
+		bytes = lxc_read_nointr(pipefd[0], buf, buf_size - 1);
 		if (bytes > 0)
 			buf[bytes - 1] = '\0';
 	}
@@ -2297,11 +2419,36 @@ int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 	return fret;
 }
 
+char *must_concat(const char *first, ...)
+{
+	va_list args;
+	char *cur, *dest;
+	size_t cur_len, it_len;
+
+	dest = must_copy_string(first);
+	cur_len = it_len = strlen(first);
+
+	va_start(args, first);
+	while ((cur = va_arg(args, char *)) != NULL) {
+		it_len = strlen(cur);
+
+		dest = must_realloc(dest, cur_len + it_len + 1);
+
+		(void)memcpy(dest + cur_len, cur, it_len);
+		cur_len += it_len;
+	}
+	va_end(args);
+
+	dest[cur_len] = 0;
+	return dest;
+}
+
 char *must_make_path(const char *first, ...)
 {
 	va_list args;
 	char *cur, *dest;
 	size_t full_len = strlen(first);
+	size_t buf_len;
 
 	dest = must_copy_string(first);
 
@@ -2310,10 +2457,13 @@ char *must_make_path(const char *first, ...)
 		full_len += strlen(cur);
 		if (cur[0] != '/')
 			full_len++;
-		dest = must_realloc(dest, full_len + 1);
+
+		buf_len = full_len + 1;
+		dest = must_realloc(dest, buf_len);
+
 		if (cur[0] != '/')
-			strcat(dest, "/");
-		strcat(dest, cur);
+			(void)strlcat(dest, "/", buf_len);
+		(void)strlcat(dest, cur, buf_len);
 	}
 	va_end(args);
 
@@ -2326,21 +2476,21 @@ char *must_append_path(char *first, ...)
 	size_t full_len;
 	va_list args;
 	char *dest = first;
+	size_t buf_len;
 
 	full_len = strlen(first);
 	va_start(args, first);
 	while ((cur = va_arg(args, char *)) != NULL) {
 		full_len += strlen(cur);
-
 		if (cur[0] != '/')
 			full_len++;
 
-		dest = must_realloc(dest, full_len + 1);
+		buf_len = full_len + 1;
+		dest = must_realloc(dest, buf_len);
 
 		if (cur[0] != '/')
-			strcat(dest, "/");
-
-		strcat(dest, cur);
+			(void)strlcat(dest, "/", buf_len);
+		(void)strlcat(dest, cur, buf_len);
 	}
 	va_end(args);
 
@@ -2353,6 +2503,7 @@ char *must_copy_string(const char *entry)
 
 	if (!entry)
 		return NULL;
+
 	do {
 		ret = strdup(entry);
 	} while (!ret);
@@ -2393,6 +2544,18 @@ bool has_fs_type(const char *path, fs_type_magic magic_val)
 	return has_type;
 }
 
+bool fhas_fs_type(int fd, fs_type_magic magic_val)
+{
+	int ret;
+	struct statfs sb;
+
+	ret = fstatfs(fd, &sb);
+	if (ret < 0)
+		return false;
+
+	return is_fs_type(&sb, magic_val);
+}
+
 bool lxc_nic_exists(char *nic)
 {
 #define __LXC_SYS_CLASS_NET_LEN 15 + IFNAMSIZ + 1
@@ -2417,8 +2580,11 @@ bool lxc_nic_exists(char *nic)
 int lxc_make_tmpfile(char *template, bool rm)
 {
 	int fd, ret;
+	mode_t msk;
 
+	msk = umask(0022);
 	fd = mkstemp(template);
+	umask(msk);
 	if (fd < 0)
 		return -1;
 
@@ -2519,7 +2685,8 @@ int lxc_set_death_signal(int signal)
 	int ret;
 	pid_t ppid;
 
-	ret = prctl(PR_SET_PDEATHSIG, signal, 0, 0, 0);
+	ret = prctl(PR_SET_PDEATHSIG, prctl_arg(signal), prctl_arg(0),
+		    prctl_arg(0), prctl_arg(0));
 
 	/* Check whether we have been orphaned. */
 	ppid = (pid_t)syscall(SYS_getppid);
@@ -2550,3 +2717,85 @@ void remove_trailing_newlines(char *l)
 	while (--p >= l && *p == '\n')
 		*p = '\0';
 }
+
+int fd_cloexec(int fd, bool cloexec)
+{
+	int oflags, nflags;
+
+	oflags = fcntl(fd, F_GETFD, 0);
+	if (oflags < 0)
+		return -errno;
+
+	if (cloexec)
+		nflags = oflags | FD_CLOEXEC;
+	else
+		nflags = oflags & ~FD_CLOEXEC;
+
+	if (nflags == oflags)
+		return 0;
+
+	if (fcntl(fd, F_SETFD, nflags) < 0)
+		return -errno;
+
+	return 0;
+}
+
+int recursive_destroy(char *dirname)
+{
+	int ret;
+	struct dirent *direntp;
+	DIR *dir;
+	int r = 0;
+
+	dir = opendir(dirname);
+	if (!dir)
+		return -1;
+
+	while ((direntp = readdir(dir))) {
+		char *pathname;
+		struct stat mystat;
+
+		if (!strcmp(direntp->d_name, ".") ||
+		    !strcmp(direntp->d_name, ".."))
+			continue;
+
+		pathname = must_make_path(dirname, direntp->d_name, NULL);
+
+		ret = lstat(pathname, &mystat);
+		if (ret < 0) {
+			if (!r)
+				WARN("Failed to stat \"%s\"", pathname);
+
+			r = -1;
+			goto next;
+		}
+
+		if (!S_ISDIR(mystat.st_mode))
+			goto next;
+
+		ret = recursive_destroy(pathname);
+		if (ret < 0)
+			r = -1;
+
+	next:
+		free(pathname);
+	}
+
+	ret = rmdir(dirname);
+	if (ret < 0) {
+		if (!r)
+			SYSWARN("Failed to delete \"%s\"", dirname);
+
+		r = -1;
+	}
+
+	ret = closedir(dir);
+	if (ret < 0) {
+		if (!r)
+			SYSWARN("Failed to delete \"%s\"", dirname);
+
+		r = -1;
+	}
+
+	return r;
+}
