@@ -31,6 +31,7 @@
 #include "confile_utils.h"
 #include "lxc/state.h"
 #include "lxctest.h"
+#include "utils.h"
 
 static int set_get_compare_clear_save_load(struct lxc_container *c,
 					   const char *key, const char *value,
@@ -302,7 +303,7 @@ int main(int argc, char *argv[])
 	char tmpf[] = "lxc-parse-config-file-XXXXXX";
 	char retval[4096] = {0};
 
-	fd = mkstemp(tmpf);
+	fd = lxc_make_tmpfile(tmpf, false);
 	if (fd < 0) {
 		lxc_error("%s\n", "Could not create temporary file");
 		exit(fret);
